#include "glRenderer.bi"  

' In this example I create an rigid body as an 1m x 2m x 3m box with 7 commands. 
' (in next example I show you the same result with one command)

'test of:
' SimulatorCreateRigidBody()
' RigidBodyAddGeometry()
' GeometrySetBoxSize()
' BoxInertiaTensorV3()
' RigidBodySetInertiaTensorV3()
' RigidBodySetMass()
' RigidBodyUpdateBoundingInfo()

' RigidBodySetPosition()

' Renderer.LookAt()
' Renderer.Render(neRigidBody)

'
' main
'
' create the renderer
var pgr = glRenderer()

WindowTitle "file: '20_neRigidBody_classic.bas'"

' create the physics simulator
var sim = CreateSimulator()

' a simple ridid body box in 7 steps (the classic way)
' step 1: create a rigid body
var body = SimulatorCreateRigidBody(sim)
' step 2:  add a shapeless collision geometry to the body (you can add more than one collison shapes to a body)
var geo = RigidBodyAddGeometry(body)
' step 3: set the collision shape of the geometry as a 1m x 2m x 3m box
GeometrySetBoxSize(geo, 1,2,3)
' step 4: calculate the a box inertia tensor by size and 10Kg mass 
var tensor = BoxInertiaTensorV3(1,2,3, 10)
' step 5: set the inertia tensor 
RigidBodySetInertiaTensorV3(body, tensor)
' step 6: set the rigid body mass in Kg
RigidBodySetMass(body, 10)
' step 7: tell the body we are fin with adding stuff (update the internal state of the rigid body)
RigidBodyUpdateBoundingInfo(body)

' move it 10m up
RigidBodySetPosition(body, 0,10,0)

' create a 200m x 2m x 200m (static) box as ground
var ground = CreateAnimatedBodyBox(sim, 200,2,200)
AnimatedBodySetPosition(ground, 0,-1,0)

' set the camera position (3m up and 10m back)
pgr.LookAt(0,3,10)
dim as string text
' main loop
while inkey() = ""
  ' advance the simulation one step in time
  SimulatorAdvance(sim)
  ' clear the render canvas
  pgr.cls
  ' render the ranimated body (ground)
  pgr.render(ground)
  ' render the rigid body box 
  pgr.render(body)
  ' draw the frame rate on canvas
  text = "fps: " & pgr.FPS & iif(SimulatorGetIdleBodyCount(sim)>0," Be quiet the cute box is sleeping now !","")
  pgr.Text(0,0, text)

  ' show the canvas on screen
  pgr.show()
  ' advance the renderer one frame in time
  pgr.Advance()  
wend
' (optional but clean coding style)
' destroy the instance of the simulator and all used resoures
DestroySimulator(sim)
