#include "glRenderer.bi" 

' In this file I show you how easy you can extend the neTriangleMesh class 
#include "neSandbox.bi"

' test of:

' CreateSimulatorEx()
'  class neSandbox
'  SimulatorSetTerrainMesh(neSandbox)
'  Render(neSandbox)

'
' main
'
const MAX_RIGID_BODIES = 100 ' default is 50 !
dim as neRigidBody ptr bodies(MAX_RIGID_BODIES-1)

' create the [p]hysics [g]raphic [r]enderer (with default settings)
var pgr = glRenderer()

windowtitle "create neSimulator ..."
' create the simulator with our number of rigid bodies (default = 50)
var sim = CreateSimulatorEx(MAX_RIGID_BODIES)

windowtitle "create random neSandbox ..."
var sandBox = new neSandBox()
' get current sandBox dimensions
var dims = V3Sub(sandBox->maxBound,sandBox->minBound)

windowtitle "set neSandBox as neTerrain ..."
SimulatorSetTerrainMesh(sim, sandBox)

windowtitle "prepare neTriangleMesh for rendering ..."
pgr.SetMesh(sandBox)

windowtitle "create some rigid bodies ..."

for i as integer = 0 to ubound(bodies)
  select case as const i and 3
  case 0 : bodies(i) = CreateRigidBodySphere(sim,neRND(0.5,1.5),100)
           RigidBodySetSleepingParameter(bodies(i),.2f)
           RigidBodySetAngularDamping(bodies(i),.5f)
           RigidBodySetLinearDamping(bodies(i),.1f)
  case 1 : bodies(i) = CreateRigidBodyCapsule(sim)           
  case 2 : bodies(i) = CreateRigidBodyCube(sim)            
  case 3 : bodies(i) = CreateRigidBodyBox(sim,0.5f,1.0f,2.0f)           
  end select  
    
  dim as single xp = neRnd(-.4f,.4f)*dims.x
  dim as single yp = dims.y
  dim as single zp = neRnd(-.4f,.4f)*dims.z
  RigidBodySetPositionRotation(bodies(i), xp,yp,zp, rnd*359,rnd*359,rnd*359)
next

windowtitle "file: '31_nesandBox.bas'"

dim as neV3 observer
dim as single w

while inkey()=""
  ' advance the simulation one timestep in time
  SimulatorAdvance(sim) ' 60 Hz
  ' advance the render one frame in time
  pgr.Advance()
  
  if (pgr.frame and 1) then ' 30 Hz
   ' only for fun rotate the observer around the center
    with observer
      .x = neCos(w)*dims.x
      .y = dims.y*3 + dims.y*4 + neSin(w*2)*dims.y*4
      .z = neSin(w)*dims.z
      
    end with  
    ' advance the rotation (15 degree per seconds)
    w += Deg2Rad*15*pgr.Timescale
    
    ' look from observer position to center of the mesh
    pgr.LookAt(observer)
    
    ' clear the render canvas
    pgr.Cls
    
    ' render the sandBox
    pgr.Render(sandBox)
    
    ' render the bodies
    for i as integer = 0 to ubound(bodies)
      pgr.Render(bodies(i))
    next
    
    ' output some infos
    pgr.Text(0,0,"frame: " & pgr.Frame & _
                 " fps: " & pgr.FPS & _
                 " idle bodies: " & SimulatorGetIdleBodyCount(sim) & _
                 " / " & MAX_RIGID_BODIES)
    
    ' show canvas on screen
    pgr.Show
  end if
wend
' optional but clean coding style
DestroySimulator(sim)

