#include "glRenderer.bi"
#include "neGround.bi"

' A wall made of stacking bricks !

' test of stacking bricks 
const MAX_BRICKS  = 35*35
const MAX_SPHERES = 100
const as single SPHERE_RADIUS = .25f
const WALL_ZPOS   = 0
dim as integer SQR_BRICKS = neSqr(MAX_BRICKS)


var gfx = glRenderer()
var lookFrom = neV3(0, SQR_BRICKS/2,SQR_BRICKS*2)
var lookTo   = neV3(0, SQR_BRICKS/2, 0)

gfx.LookAt(lookFrom,lookTo)

dim as neSimulatorSizeInfo SizeInfo
with SizeInfo
  .animatedBodiesCount  = 0 ' no static bodies
  .controllersCount     = 0 ' no rigid or joint controllers
  .constraintsCount     = 0 ' no joints
  .sensorsCount         = 0 ' no sensors
  .rigidParticleCount   = 0 ' no particles
  .rigidBodiesCount     = MAX_BRICKS + MAX_SPHERES
  .geometriesCount      = .rigidBodiesCount
  .overlappedPairsCount = (.geometriesCount*(.geometriesCount-1))/2
  ' !!! 20,000,000 !!!
  if .overlappedPairsCount>20000000 then .overlappedPairsCount=20000000
end with
var sim = CreateSimulator(SizeInfo)

' the static ground
var ground = new neGround()
SimulatorSetTerrainMesh(sim, ground)

' create a wall of stacking bricks
var rowLeft = -SQR_BRICKS\2
var tWall=timer()
dim as neRigidBody ptr bricks(MAX_BRICKS-1)
for index as integer = 0 to ubound(bricks)
  var y = int(index  \  SQR_BRICKS)
  var x = int(index mod SQR_BRICKS)
  bricks(index) = CreateRigidBodyCube(sim,1,10)
  RigidBodySetAngularDamping(bricks(index),.7)
  RigidBodySetLinearDamping (bricks(index),.7)
  RigidBodySetPosition(bricks(index),rowLeft + x + (y and 1)*0.5,.5+y,WALL_ZPOS)
 
  if (index+1) mod SQR_BRICKS=0 then
    SimulatorAdvance(sim)
    gfx.Advance()
    gfx.cls()
    gfx.Render(ground)
#ifndef NO_GROUND_SHADOW  
    gfx.GroundShadow(true)
      for j as integer = 0 to index-1 : gfx.Render(bricks(j))  : next
    gfx.GroundShadow(false) 
#endif  
    for j as integer = 0 to index-1 : gfx.Render(bricks(j)) : next
    gfx.text(0,0,"add brick " & index & "/" & MAX_BRICKS & " to the wall")
    gfx.show()
  end if  
  if asc(inkey())=27 then beep : end 1
next

var c=0
while SimulatorGetIdleBodyCount(sim)<MAX_BRICKS
  SimulatorAdvance(sim)
  c+=1
  if c mod 10=0 then
    gfx.Advance()
    gfx.cls()
    gfx.Render(ground)
#ifndef NO_GROUND_SHADOW
    gfx.GroundShadow(true)
      for i as integer = 0 to ubound(bricks) : gfx.Render(bricks(i)) : next
    gfx.GroundShadow(false) 
#endif    
    for i as integer = 0 to ubound(bricks) : gfx.Render(bricks(i)) : next
    gfx.Text(0,0,"Wait all bricks must become idle " & SimulatorGetIdleBodyCount(sim) & " / " & MAX_BRICKS)
    gfx.show()
  end if  
  if asc(inkey())=27 then beep : end 1
wend
' reset damping 
for i as integer = 0 to MAX_BRICKS-1
  RigidBodySetAngularDamping(bricks(i),0.0)
  RigidBodySetLinearDamping(bricks(i),0.0)
next


dim as neRigidBody ptr spheres(MAX_SPHERES-1)
dim as neV3 spherePosition
for i as integer = 0 to ubound(spheres)
  spheres(i) = CreateRigidBodySphere(sim,SPHERE_RADIUS*2,100)
  RigidBodySetActive(spheres(i),false)
  RigidBodyGravityEnable(spheres(i),false)
  spherePosition.x = 0
  spherePosition.y = SPHERE_RADIUS + (SPHERE_RADIUS*2*i)
  spherePosition.z = -200 ' behind the wall trigger first shot
  RigidBodySetPositionV3(spheres(i),spherePosition)  
next

var lastFrame=0
var w = NE_PI*0.5
' The render was used while building the wall
' time to reset the renderer state (timeScale,fps,frame counter ...)
gfx.Reset() 

while inkey()=""
  SimulatorAdvance(sim)
  gfx.Advance()
  
  ' only for fun rotate the camera around the wall
  lookFrom.x=neCos(w)*SQR_BRICKS*2
  lookFrom.z=neSin(w)*SQR_BRICKS*2
  w += DEG2RAD*5*gfx.timeScale
  gfx.LookAt(lookFrom,lookTo)

  gfx.cls
  gfx.render(ground)

#ifndef NO_GROUND_SHADOW
  gfx.GroundShadow(true)
    for i as integer = 0 to ubound(spheres) : gfx.Render(spheres(i)) : next
    for i as integer = 0 to ubound( bricks) : gfx.Render(bricks(i))  : next
  gfx.GroundShadow(false)  
#endif

  for i as integer = 0 to ubound(spheres) : gfx.Render(spheres(i)) : next
  for i as integer = 0 to ubound( bricks) : gfx.Render(bricks(i))  : next
  
  gfx.Text(0,0,"fps: " & gfx.fps & " frame: " & gfx.frame)
  gfx.Show
 
  ' allow 120 shots per minute (every 2 seconds)!
  const shotsPerMinute = 120 ' (try 600=10 per seconds if you like)
  if (gfx.Frame-lastFrame)>(gfx.fps/(shotsPerMinute/60)) then
    lastFrame=gfx.frame
    ' search a free bullet
    dim as integer foundSphere=-1
    for i as integer = 0 to ubound(spheres)
      spherePosition = RigidBodyGetPositionV3(spheres(i))
      if spherePosition.z>-200 andalso spherePosition.z<0 then 
        if RigidBodyGravityIsOn(spheres(i))=false then 
          RigidBodyGravityEnable(spheres(i),true)
        end if
      end if  
      if spherePosition.z<-190 then foundSphere=i
    next
    
    if foundSphere>-1 then
      dim as neV3 targetDirection,targetPosition      
      ' create a new shot select the highest sleeping/idle brick as target
      ' NOTE: the goal isn't to destroy the wall as fast as posible !
      targetPosition.y=0.25
      ' search a brick as new target
      dim as integer foundBrick=-1
      for i as integer = ubound(bricks) to 0 step -1
        ' is sleeping ?
        if RigidBodyIsIdle(bricks(i)) then
          ' not marked as target from other shoot before ?
          if RigidBodyGetUserData(bricks(i))=0 then
            var brickPosition = RigidBodyGetPositionV3(bricks(i))
            ' is the brick a part of the wall
            if (brickPosition.y>targetPosition.y) andalso (brickPosition.z>-.5) andalso (brickPosition.z<.5) then
              targetPosition = brickPosition
              foundBrick=i:exit for
            end if  
          end if  
        end if
      next
      
      ' we found a sleeping brick as a part of the wall
      if foundBrick>-1 then
        targetPosition.y+=SPHERE_RADIUS
        ' reset sphere properties for a new shot
        spherePosition = neV3(0,SPHERE_RADIUS+0.01,100)
        RigidBodySetAngularMomentumV3(spheres(foundSphere),neV3.Zero)
        RigidBodySetVelocityV3       (spheres(foundSphere),neV3.Zero)
        RigidBodySetTorqueV3         (spheres(foundSphere),neV3.Zero)
        RigidBodySetPositionV3       (spheres(foundSphere),spherePosition)
        RigidBodyGravityEnable(spheres(foundSphere),false)
        if RigidBodyGetActive(spheres(foundSphere))=false then
          RigidBodySetActive(spheres(foundSphere),true)
        end if
        ' mark the brick as target
        RigidBodySetUserData(bricks(foundBrick),cptr(any ptr,1))
        ' direction of the shot = normalized(brickPosition - spherePosition)
        targetDirection = V3Normalized(V3Sub(targetPosition,spherePosition))
        targetDirection=V3Mul(targetDirection,30) ' use the normalized direction vector * 30 as bullet velocity vector
        RigidBodySetVelocityV3(spheres(foundSphere),targetDirection)        
      end if
    end if 
  end if 
wend
DestroySimulator(sim)
