#include "glRenderer.bi" 

' a chain of rigidbodies

const N_RB = 100         ' number of rigid bodies
const N_AB = 1           ' number of animated bodies (the ground box)
const N_GE = N_RB + N_AB ' number of geometries
const N_JO = N_RB        ' number of joints

dim as neSimulatorSizeInfo SizeInfo
with SizeInfo
  .rigidBodiesCount     = N_RB ' alloc # rigid bodies
  .animatedBodiesCount  = N_AB ' alloc # animated bodies
  .geometriesCount      = N_GE ' alloc # geometries
  .sensorsCount         = 0    ' alloc # sensors 
  .constraintsCount     = N_JO ' alloc # joints
  .rigidParticleCount   = 0    ' alloc # particles
  .controllersCount     = 0    ' alloc # controllers
  .overlappedPairsCount = (.geometriesCount*(.geometriesCount-1))/2
end with

dim as neRigidBody ptr bodies(N_RB-1)
dim as neJoint     ptr joints(N_JO-1)

var sim = CreateSimulator(SizeInfo)


var ground = CreateAnimatedBodyBox(sim, 10000,2,10000)
AnimatedBodySetPosition(ground,0,-1,0)

const as single SPHERE_DISTANCE = 1
const as single SPHERE_DIAMETER = SPHERE_DISTANCE-0.1


for i as integer = 0 to ubound(bodies)
  bodies(i) = CreateRigidBodySphere(sim,SPHERE_DIAMETER,100)
  ' allow collision by indirectly connected bodies (connected via joint)
  RigidBodySetCollideConnected(bodies(i),true)
  RigidBodySetPosition(bodies(i),i*SPHERE_DISTANCE,ubound(bodies)*SPHERE_DISTANCE,0)
  if i=0 then ' the first sphere are connected via a joint to a nail in the world :-)
    joints(i) = SimulatorCreateRigidBodyWorldJoint(sim,bodies(i))
  else        ' all other spheres conntected to the last sphere
    joints(i) = SimulatorCreateRigidBodyRigidBodyJoint(sim,bodies(i),bodies(i-1))
  end if  
  JointSetType(joints(i),NE_BALLSOCKET)
  JointSetPositionRotationFrameWorld(joints(i), i*SPHERE_DISTANCE,ubound(bodies)*SPHERE_DISTANCE,0, 0,0,0)
  ' optional enable damping of forces acting on the joints
  'JointSetDampingFactor(joints(i),0.5)
  JointEnable(joints(i),true)
next

var gfx = glRenderer()
var lookFrom = neV3(0,ubound(bodies)*SPHERE_DISTANCE/2,ubound(bodies)*SPHERE_DISTANCE*1.25)
var lookTo   = neV3(0,ubound(bodies)*SPHERE_DISTANCE/2,0)
gfx.LookAt(lookFrom, lookTo) 

WindowTitle "file: '51_Ball_Socket_neJoints.bas'"
dim as boolean bExit,bDrawJoints
while not bExit
  var key = asc(inkey())
  if key=asc("j") then
    bDrawJoints = not bDrawJoints
  elseif key=27 then
    bExit = true
  end if  
    
  SimulatorAdvance(sim)
  gfx.Advance()
  gfx.Cls
  gfx.Render(ground)
  if bDrawJoints then
    for i as integer =0 to ubound(bodies)
      gfx.Render(joints(i))
    next
  else
    for i as integer =0 to ubound(bodies)
      gfx.Render(bodies(i))    
    next
  end if
  gfx.Text(0,0,"press [J] toggle joint drawing fps: " & gfx.fps)
  gfx.Show()
wend

DestroySimulator(sim)

