#include "glRenderer.bi"

const as integer N_BO = 20        ' number of boxes
const as integer N_SP = 10        ' number of spheres
const as integer N_JO = N_BO-1    ' number of joints (between the rigid bodies)
const as integer N_RB = N_BO+N_SP ' number of rigid bodies (boxes + spheres)
const as integer N_AB =  1        ' number of animated bodies (the static ground)
const as integer TOTALBODIES = N_RB + N_AB

const as single DIAMETER = 5.0f   ' sphere diameter
const as single MASS = 1.0f

dim as neSimulatorSizeInfo SizeInfo
dim as neRigidBody    ptr Boxes(N_BO-1)    
dim as neJoint        ptr Joints(N_JO-1)  
dim as neRigidBody    ptr Spheres(N_SP-1) 
dim as neAnimatedBody ptr Ground



' Create and set simulator properties
with SizeInfo
  .RigidBodiesCount    = N_RB  ' RigidBodies
  .AnimatedBodiesCount = N_AB  ' AnimatedBodies
  .ConstraintsCount    = N_JO  ' Joints
  .GeometriesCount     = TOTALBODIES
  .OverlappedPairsCount= (TOTALBODIES*(TOTALBODIES-1))\2
end with

' Create Simulator
var sim = CreateSimulator(SizeInfo) 

' Create Rigid Bodies (boxes)
dim as neV3 BoxSize = neV3(10,10,1)


for i as integer = 0 to ubound(boxes)
  Boxes(i) = CreateRigidBodyBox(Sim,BoxSize.x,BoxSize.y,BoxSize.z, MASS)
  RigidBodySetPosition(Boxes(i),N_BO*-BoxSize.x*0.5 + i*BoxSize.x,20.0,0.0)
next

' Create rigid bodies (spheres)
for i as integer = 0 to ubound(Spheres)
  Spheres(i) = CreateRigidBodySphere(Sim,DIAMETER, MASS)
  RigidBodySetPosition(Spheres(i),N_BO*-BoxSize.x*0.5 + i*BoxSize.x*2,DIAMETER*0.5,neRnd(-.1,.1))
next

' Create an animated body for the floor
Ground = CreateAnimatedBodyBox(Sim, 1000,2,1000)
AnimatedBodySetPosition(Ground,0,-1,0)

' create hinge Joint between boxes(i) and boxes(i+1)
for i as integer = 0 to ubound(joints)
  Joints(i)=SimulatorCreateRigidBodyRigidBodyJoint(Sim,Boxes(i),Boxes(i+1))
  JointSetType(Joints(i),NE_HINGE)
  var a = RigidBodyGetPositionV3(Boxes(i))
  var b = RigidBodyGetPositionV3(Boxes(i+1))
  var p = V3Lerp(a,b,0.5) '= a + (b-a)*0.5
  JointSetPositionRotationFrameWorldV3(Joints(i),p)
  JointSetJointLength(Joints(i),BoxSize.y)
  JointSetLowerLimit(Joints(i),NE_PI*-0.25)
  JointSetUpperLimit(Joints(i),NE_PI* 0.25)
  JointLimitEnable(Joints(i),True)
  'JointSetDampingFactor(Joints(i),20)
  JointEnable(Joints(i),True)
next

' note: the boxes and joints was defined so that the hinge Y axis points up !
' I rotated all boxes 90 degrees around the x axis to get the horizontal result. 
for i as integer = 0 to ubound(boxes)
  RigidBodySetRotation(Boxes(i),90,0,0)
  RigidBodySetCollideConnected(Boxes(i),True)
next

var gfx = glRenderer()
gfx.LookAt(0,17,200) 

while inkey()=""
  SimulatorAdvance(sim) ' 60 Hz.
  gfx.Advance()
  
  gfx.Cls
  
  gfx.Render(ground)

  gfx.GroundShadow(true)
    for i as integer = 0 to ubound(boxes)
      gfx.Render(boxes(i))
    next  
    for i as integer = 0 to ubound(spheres)
      gfx.Render(spheres(i))
    next
  gfx.GroundShadow(false)    
  
  for i as integer = 0 to ubound(boxes)
    gfx.Render(boxes(i))
  next  
  for i as integer = 0 to ubound(spheres)
    gfx.Render(spheres(i))
  next
  gfx.Text(0,0,"fps: " & gfx.fps & " frame: " & gfx.frame)
  gfx.Show
wend

DestroySimulator(sim)

