#include "glRenderer.bi" 

' test of:

' RigidBodyAddSensor()
' RigidBodyBeginIterateSensor()
' RigidBodyGetNextSensor()

' RigidBodySetUserData()
' RigidBodyGetUserData()

' RigidBodyAddController()
' the implementation of an neRigidBodyControllerCallback() 
' RigidBodyControllerGetRigidBody()
' SensorGetDetectDepth()
' SensorGetLinePosition()

' This rigid body controller callback are called by the simulator 
' if the attached sensor collided with any other active collision shape.
' If the penetration depth value of an sensor is >0.0 then the sensor collided.
sub RigidBodyControllerCallback cdecl (byval controller as neRigidBodyController ptr, byval DT as single)
  var body = RigidBodyControllerGetRigidBody(controller)
  dim as single ptr pDepth = RigidBodyGetUserData(body)
  var mass = RigidBodyGetMass(body)
  ' in this simple example only one sensor was attached to the body
  ' but it shows you how to iterate to all sensors (if more than one)
  ' set the iterator to the first sensor
  RigidBodyBeginIterateSensor(body)
  ' get first sensor
  var sensor  = RigidBodyGetNextSensor(body)
  ' loop over all sensors
  while sensor<>NULL
    var depth = SensorGetDetectDepth(sensor)
    ' share the depth value with the main program
    if pDepth<>NULL then *pDepth=Depth
    ' only if the penetration depth> 0.0
    if depth>0 then
      depth*=0.4
      var vPos = SensorGetLinePositionV3(sensor)
      'RigidBodyApplyImpulse(body, vPos.x,depth*mass*9.81*2*DT, vPos.z)
      RigidBodyApplyImpulse(body, 0,depth*mass*9.81*2*DT,0)
    end if
    ' get next senor (if any) 
    sensor = RigidBodyGetNextSensor(body) 
  wend
end sub


'
' main
'
var pgr = glRenderer()
WindowTitle "file: '60_sensor.bas'"

var sim = CreateSimulator()

' create a 10m x 2m x 10m (static) box as ground
var ground = CreateAnimatedBodyBox(sim, 10.0f, 2.0f, 10.0f)
AnimatedBodySetPosition(ground, 0.0f, -1.0f, 0.0f)

' create a cube 1m x 1m x 1m with 50KG mass.
var body = CreateRigidBodyCube(sim,1,50)
' Add a line sensor to the box
var sensor = RigidBodyAddSensor(body)
' Set position and direction of the line sensor
SensorSetLineSensor(sensor, 0,0,0, 0,-2,0)
RigidBodyUpdateBoundingInfo(body)

' damp the rotation of the body
' (comment it out and see what happent)
RigidBodySetAngularDamping(body,.9) 

RigidBodySetPosition(body, 0.0f, 1.0f, 0.0f)

' To get any sensor collision data we must add a neRigidBodyController
' This controller will call a user defined neRigidBodyControllerCallback 
' for any collision between the sensor and other collision shapes.
var RigidBodyController = RigidBodyAddController(body, @RigidBodyControllerCallback)

' NOTE: All collision data of a sensor can "ONLY BE READ INSIDE" of the controller callback !
' To share collision data between the callback and the main program you can use shared global vars
' but I self don't like global vars at all so I use the neClass "user data" property instead.

' You can read and write the "user data" property of this classes:
' neAnimatedBody, neRigidBody, neGeometry, neTriangle, neSensor and neJoint.

' As an simple example I will show you how to read the depth of the line sensor collision
' inside the controller callback and share it with the main program.
dim as single sensorDepth : RigidBodySetUserData(body, @sensorDepth)
' Inside the callback I use RigidBodyGetUserData() and write the depth value.

const bGroundShadow = true
const bRenderSensor = true

' set the camera position (1.7m up and 5m back)
pgr.LookAt(0.0f, 1.7f, 5.0f)

' main loop
while inkey() = ""
  SimulatorAdvance(sim)
  pgr.Cls
  pgr.Render(ground)
  
  if bGroundShadow then
    pgr.GroundShadow(true)
    pgr.Render(body, bRenderSensor)
    pgr.GroundShadow(false)
  end if  

  pgr.Render(body, bRenderSensor)
  
  pgr.Text(0, 0,"fps: " & pgr.FPS)
  ' show the shared collision depth value
  pgr.Text(0,16,"sensor collision depth " & sensorDepth)
  pgr.Show

  pgr.Advance
 
wend
DestroySimulator(sim)
