#pragma once
#lang "fb"

#include "neRenderer.bi"

#include "GL/gl.bi"
#include "fbgfx.bi"
using FB

#ifdef __FB_WIN32__
  const as string SWAP_EXT  = "WGL_EXT_swap_control"
  const as string SWAP_PROC = "wglSwapIntervalEXT"
  type tSwapInterval as function stdcall (byval interval as long) as ubyte
#else
  const as string SWAP_EXT  = "GLX_EXT_swap_control"
  const as string SWAP_PROC = "glXSwapIntervalEXT"
  type tDisplay as any
  type tDrawable as long
  type tSwapInterval as sub cdecl (byval dpy as tDisplay ptr, byval drawable as tDrawable, byval interval as long)
  extern "C" lib "GL"
    declare function glXGetCurrentDisplay() as tDisplay ptr
    declare function glXGetCurrentDrawable() as tDrawable
    'declare function glXQueryExtensionsString(byval dpy as tDisplay ptr, byval nScreen as long=0) as const zstring ptr
  end extern
#endif


type glRenderer extends neRenderer
' #####################
' # plublic interface #
' #####################
  public:
  declare destructor
  declare constructor(byval iWidth  as integer=-1, _
                      byval iHeight as integer=-1, _
                      byval bFullscreen as boolean = false)
  declare sub Reset
  declare sub VSync       (byval bOnOff as boolean)
  declare sub GroundShadow(byval bOnOff as boolean)
  declare sub Mode2D      (byval bOnOff as boolean)

  declare sub LookAt(byval ex as single=0.0f, byval ey as single=1.7f, byval ez as single=-1.0f, _
                     byval lx as single=0.0f, byval ly as single=0.0f, byval lz as single=0.0f, _
                     byval ux as single=0.0f, byval uy as single=1.0f, byval uz as single=0.0f)

  declare sub LookAt(byref e as const neV3, _
                     byref t as const neV3=neV3.Zero, _
                     byref u as const neV3=neV3.Up)

  declare function CanvasWidth  as integer
  declare function CanvasHeight as integer
  declare sub      Advance(byval bSleep as boolean=true)
  declare function FPS       as integer
  declare function Frame     as integer
  declare function Runtime   as double
  declare function Timescale as double
  
  declare sub SetMesh(byval mesh as neTriangleMesh ptr)
  
  declare sub Render(byval body as neAnimatedBody ptr)
  declare sub Render(byval body as neRigidBody ptr)
  declare sub Render(byval mesh as neTriangleMesh ptr)
  
  declare sub Render(byval joint as neJoint ptr)
  declare sub Render(byval body as neRigidBody ptr, byval bDrawSensors as boolean)
  
 
  declare sub Show
  declare sub Cls(byval bColorBuffer   as boolean = true, _
                  byval bDepthBuffer   as boolean = true, _
                  byval bStencilBuffer as boolean = false)
                  
  
  
  declare sub Text(byval x as integer, byval y as integer, _
                   byref txt as string, _
                   byval r as single = 1.0f, _
                   byval g as single = 1.0f, _
                   byval b as single = 1.0f)

  declare sub setClearColor(byval r as single = .4f, _
                            byval g as single = .4f, _
                            byval b as single = .9f)
  declare sub setAlphaValue(byval a as single)
' ###########                            
' # private #
' ###########
  private:
  declare sub privPerspective(byval fov as single=60.0f, byval ratio as single=-1.0f, byval zNear as single=1.0f,byval zFar as single=8192.0f)  
  declare sub privRender(byval _body as neAnimatedBody ptr, _
                         byval r as single, _
                         byval g as single, _
                         byval b as single)
  declare sub privRender(byval _body as neRigidBody ptr, _
                         byval r as single, _
                         byval g as single, _
                         byval b as single, _
                         byval bDrawSensors as boolean)
  declare sub privRender(byval geom as neGeometry ptr)
  declare sub privRender(byval sens as neSensor ptr)
  declare sub privSetLight()
  
  declare function privCreateCube   as GLuint
  declare function privCreateSphere as GLuint
  declare sub      privCreateCapsule(byref Top as GLuint, byref Body as GLuint, byref Bottom as GLuint)
  as tSwapInterval privSwapInterval
#ifndef __FB_WIN32__
  as tDisplay ptr privDisplay
  as tDrawable    privDrawable
#endif
  as integer privWidth,privHeight,privBitsPerPixel
  as GLuint privBoxList,privSphereList,privCapsuleTop,privCapsuleBody,privCapsuleBottom,privFontTexture,privCharBase
  as single privShadowMatrix(3,3), privShadowColor(3), privLightPosition(3),privAlpha
  as boolean privIsFullscreen,privShadowIsOn,privMode2DIsOn,privAutoNormaIsOn,privVSyncIsOn,privBlendIsOn
  as integer privFPS,privFrame
  as double privStart,privNow,privLast,privScale

  as single privMat4x4(15)
  #define privMatrix @privMat4x4(0)
end type


destructor glRenderer
  if privFontTexture then 
    glBindTexture(GL_TEXTURE_2D,0)
    glDeleteTextures(1,@privFontTexture)
  end if  
  if privCharBase then glDeleteLists(privCharBase,256) 
  #define glDeleteList(list) if list then glDeleteLists(list,1)
  glDeleteList(privBoxList)
  glDeleteList(privSphereList)
  glDeleteList(privCapsuleTop)
  glDeleteList(privCapsuleBody)
  glDeleteList(privCapsuleBottom)
  #undef glDeleteList
end destructor

constructor glRenderer(byval userWidth as integer, byval userHeight as integer, byval userFullscreen as boolean) 
  screeninfo privWidth,privHeight,privBitsPerPixel

  const as integer iCharWidth=8, iCharHeight=16
  dim as integer iImageWidth =iCharWidth*256
  dim as integer iImageHeight=iCharHeight
  ' create NULL screen
  screenres iImageWidth,iImageHeight,32,,GFX_NULL
  ' set fontsize to 8 x 16
  width iImageWidth\iCharWidth, iImageHeight\iCharHeight 
  ' create font image
  var FontImage = ImageCreate(iImageWidth,iImageHeight,0)
  if FontImage=NULL then
    print "fatal error: can't create Image !"
    beep : sleep : end 1
  end if    
  
  ' draw 255 chars in the font image
  for char as integer = 0 to 255
    draw string FontImage,(char*iCharWidth,0),chr(char),&HFFFFFFFF
  next
  dim as integer iImagePitch,iTexturePitch,iDummy
  dim as ubyte ptr ImagePixels,ImageRow,TexturePixels,TextureRow
  ' get pixels and pitch from font image
  if ImageInfo(FontImage,iDummy,iDummy,iDummy,iImagePitch,ImagePixels)<>0 or ImagePixels=NULL then
    print "fatal error: can't retrive Image pixels !"
    beep : sleep : end 1
  end if   
  ImageRow = ImagePixels
  ' get pixels and pitch from font texture
  iTexturePitch = iImageWidth*4
  TexturePixels = allocate(iImageHeight*iTexturePitch)
  if TexturePixels=NULL then
    print "fatal error: can't allocate texture pixels !"
    beep : sleep : end 1
  end if  
  
  ' copy row by row from image pixels to texture pixels
  TextureRow = TexturePixels
  for y as integer=1 to iImageHeight
    memcpy(TextureRow,ImageRow,iTexturePitch)
    TextureRow += iTexturePitch
    ImageRow   += iImagePitch
  next
  ' delete the font image
  ImageDestroy(FontImage) : FontImage=NULL
    
  ' prepare user width and height
  if userWidth<0 orelse userHeight<0 then
    ' prepare width and height for full screen
    userWidth=privWidth : userHeight=privHeight
    ' reduce it for window
    if userFullscreen=false then userWidth*=0.75:userHeight*=0.75
  elseif userFullscreen=true then
    ' user width and height as full screen
    if (userWidth<640 orelse userHeight<480) then
      userWidth=640 : userHeight=480
    endif
  else
    ' user width and height as window 
    ' minimum is 64 x 64 maximum is desktop width x height
    if userWidth<64 then
      if userHeight>63 then
        userWidth = userHeight
      else
        userWidth=64
      endif  
    elseif userWidth>privWidth then
      userWidth=privWidth  
    endif  
    if userHeight<64 then
      if userWidth>63 then
        userHeight=userWidth
      else  
        userHeight=64
      end if  
    elseif userHeight>privHeight then
      userHeight=privHeight
    end if  
  end if
  
  var BitsPerPixel = privBitsPerPixel
  ' full screen ?
  if userFullscreen then
    var bFound = false
    var foundWidth   = userWidth
    var foundHeight  = userHeight
    var userSquared  = userWidth*userHeight
    dim as integer modeBits(3) => {32,24,16,15}
    for i as integer = 0 to ubound(modeBits)
      var mode = ScreenList(modeBits(i))
      if mode then
        foundWidth   = 4096*2
        foundHeight  = 4096
        var errorSquared = foundWidth*foundHeight
        while mode
          var fullWidth  = hiword(mode)
          var fullHeight = loword(mode)
          ' found user resolution as full screen ?
          if fullWidth=userWidth andalso fullHeight=userHeight then 
            foundWidth=fullWidth : foundHeight=fullHeight : 
            bFound = true : BitsPerPixel = modeBits(i) : exit while
          else
            ' no get nearest resolution
            var fullSquared = fullWidth*fullHeight
            if neAbs(userSquared-fullSquared) < errorSquared then
              foundWidth   = fullWidth 
              foundHeight  = fullHeight
              errorSquared = fullSquared
              bFound = true : BitsPerPixel = modeBits(i)
            end if  
          end if  
          mode = ScreenList()
        wend
      end if
      if bFound then exit for
    next
    if bFound then
      ' get exact or nearest resolution we found
      userWidth = foundWidth : userHeight = foundHeight
    else
      userFullscreen = false
    end if  
  end if   
    
  ' create OpenGL render context
  
  
  dim as integer flags=GFX_OPENGL
  if userFullscreen then
    ScreenControl(SET_GL_NUM_SAMPLES,4) 
    flags or=GFX_MULTISAMPLE
    flags or=GFX_FULLSCREEN
  end if  
  var errCode = screenres(userWidth,userHeight,BitsPerPixel,,flags)
  if errCode<>0 andalso userFullscreen=true then
    userFullscreen = false
    ' try window and desktop bits per pixel
    flags = GFX_OPENGL
    errCode = screenres(userWidth,userHeight,privBitsPerPixel,,flags)
  end if  
  
  if errCode then 
    print "fatal error: can't create OpenGL context !"
    beep : sleep : end 1
  end if  
  
  ' get current screen or window settings
  screeninfo privWidth,privHeight,privBitsPerPixel : privIsFullscreen=userFullscreen
    
  ' create and setup a 2D font texture
  glGenTextures(1, @privFontTexture)
  if privFontTexture=0 then 
    screen 0
    print "fatal error: can't create OpenGL font texture !"
    beep : sleep : end 1
  end if  
  
  glBindTexture(GL_TEXTURE_2D, privFontTexture)
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,GL_NEAREST)
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,GL_NEAREST)
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
  ' load font texture pixels in GPU memory
  glTexImage2D(GL_TEXTURE_2D,0,4,iImageWidth,iImageHeight,0,GL_RGBA,GL_UNSIGNED_BYTE,TexturePixels)
  ' delete the font texture pixels
  deallocate TexturePixels
  
  ' reserve 256 display lists (for all 256 chars)
  privCharBase = glGenLists(256)
  if privCharBase=0 then
    screen 0
    print "fatal error: can't allocate 256 OpenGL list's !"
    beep : sleep : end 1
  end if  
    
  ' create and compile all 256 char lists
  for char as integer = 0 to 255
    dim as single w = 1.0f/iImageWidth*iCharWidth
    dim as single x = char*w
    glNewList(privCharBase+char,GL_COMPILE)
      glBegin(GL_QUADS)
        glTexCoord2f(x  ,0.0f) : glVertex2i(0,0)
        glTexCoord2f(x  ,1.0f) : glVertex2i(0,iCharHeight)
        glTexCoord2f(x+w,1.0f) : glVertex2i(iCharWidth,iCharHeight)
        glTexCoord2f(x+w,0.0f) : glVertex2i(iCharWidth,0)
      glEnd()
      glTranslatef(iCharWidth,0,0)
    glEndList()
  next
  
  glClearColor(.4,.4,.9,1.0)
  glEnable(GL_LIGHTING)
  glEnable(GL_COLOR_MATERIAL)
  glCullFace(GL_BACK) : glEnable(GL_CULL_FACE)
  
  
  ' setup light and shadow matrix
  privSetLight()

  ' set canvas size
  glViewport(0, 0, privWidth, privHeight)
    
	' set 3D render mode
  glMatrixMode(GL_PROJECTION)
  privPerspective(45.0, privWidth/privHeight)
  
  glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()  
    
  glShadeModel(GL_SMOOTH) 
	glEnable(GL_DEPTH_TEST) 
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST) 
  
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
  privAlpha=1.0f
  privBlendIsOn=false
  glDisable(GL_BLEND)
  
  ' create render primitives as compiled OpenGL lists
  privBoxList      = privCreateCube()
  privSphereList   = privCreateSphere()
  privCreateCapsule(privCapsuleTop,privCapsuleBody,privCapsuleBottom)
  
  ' clear front and backbuffer
  this.cls(): flip
  this.cls(): flip
   
  ' check extensions
  dim as string privExtension
  
#ifdef __FB_WIN32__
  ScreenControl GET_GL_EXTENSIONS,privExtension
#else
  ScreenControl GET_GL_EXTENSIONS,privExtension
  privDisplay  = glXGetCurrentDisplay()
  privDrawable = glXGetCurrentDrawable()
  'privExtension = *glXQueryExtensionsString(privDisplay,0)
#endif
  if instr(privExtension,SWAP_EXT) then
    privSwapInterval = ScreenGLProc(SWAP_PROC)
  end if
  VSync(true)
  
  ' reset clock,fps and frame counter
  this.Reset()
end constructor

sub glRenderer . Reset
  privFrame = 0
  privFPS   = 60
  privScale = 1.0/privFPS
  privStart = Timer()
  privLast  = privStart
end sub

sub glRenderer . VSync(byval bOnOff as boolean)
  dim as long value = iif(bOnOff,1,0)
  if privSwapInterval = NULL then privVSyncIsOn=false : exit sub
  if bOnOff = privVSyncIsOn then exit sub

#ifdef __FB_WIN32__
  privSwapInterval(value)
  privVSyncIsOn=bOnOff 
#else
  if (privDisplay<>NULL) andalso (privDrawable<>0) then
    privSwapInterval(privDisplay,privDrawable,value)
    privVSyncIsOn=bOnOff 
  end if
#endif
end sub

sub glRenderer . setClearColor(byval r as single, _
                             byval g as single, _
                             byval b as single)
  glClearColor(r,g,b,1.0)                           
end sub                           
sub glRenderer . setAlphaValue(byval a as single)
  if a<0.0f then
    a=0.0f
  elseif a>0.99f then
    a=1.0f
  end if
  privAlpha=a
  
  if privAlpha=1.0f then
    glDisable(GL_BLEND)
    privBlendIsOn=false
  else
    glEnable(GL_BLEND)
    privBlendIsOn=true
  end if
  
end sub
sub glRenderer.LookAt(byref e as const neV3, byref t as const neV3, byref u as const neV3)
  ' get camera view direction Z "forward = normalize(eyePos - targetPos)" 
  var vForward = V3Sub(e,t)
  if vForward.x=0.0f andalso vForward.y=0.0f andalso vForward.z=0.0f then
    vForward.y=1.7
    vForward.z=-1
  end if    
  v3Normalize(vForward)
  ' get camera right vector vRight "right = normalize(cross(Y,Z))"
  var vRight = V3NormalizedCross(u,vForward)
  ' get camera up vector "up = normalize(cross(Z,X))"
  var vUp = V3NormalizedCross(vForward,vRight)
  
  ' create camera view matrix from right, up and forward vectors (X,Y,Z)
  privMat4x4( 0) = vRight.x : privMat4x4( 1) =  vUp.x : privMat4x4( 2) = vForward.x : privMat4x4( 3) = 0.0f
  privMat4x4( 4) = vRight.y : privMat4x4( 5) =  vUp.y : privMat4x4( 6) = vForward.y : privMat4x4( 7) = 0.0f
  privMat4x4( 8) = vRight.z : privMat4x4( 9) =  vUp.z : privMat4x4(10) = vForward.z : privMat4x4(11) = 0.0f

  privMat4x4(12) = 0.0f     : privMat4x4(13) =  0.0f  : privMat4x4(14) = 0.0f       : privMat4x4(15) = 1.0f
  glLoadMatrixf(privMatrix)
  ' move to the camera -position
  glTranslatef(-e.x,-e.y,-e.z)
  ' reset light position
  glLightfv(GL_LIGHT0,GL_POSITION,@privLightPosition(0))
end sub
sub glRenderer.LookAt(byval ex as single, byval ey as single, byval ez as single, _
                      byval tx as single, byval ty as single, byval tz as single, _
                      byval ux as single, byval uy as single, byval uz as single )
  LookAt(neV3(ex,ey,ez), neV3(tx,ty,tz), neV3(ux,uy,uz))
end sub  

function glRenderer.CanvasWidth  as integer
  return privWidth
end function
function glRenderer.CanvasHeight as integer
  return privHeight
end function

' Optional enable or disable a 2D render mode.
' The 2D mode is primary to output text on the canvas text(10,10,"hello world!")
' but you can draw any OpenGL primitive like points, lines, triangles, quads ... also.

' In mode 2D the coordinate system and width,height is the same as in FreeBASIC graphics mode.
' The top,left corner is (0,0) and right,bottom are (CanvasWidth-1,CanvasHeight-1).

' for example draw one white line in FreeBASIC: line (0,0) - (100,100), RGB(255,255,255)
' In OpenGL with Mode2D(true)
' glBegin(GL_LINES) 
'   glColor3ub(255,255,255) :  glVertex2f(0,0) : glVertex2f(100, 100)
' glEnd()

' if you are done with any text or primitive drawing don't forget to disable mode 2D !

sub glRenderer.Mode2D(byval bOnOff as boolean)
  if bOnOff then
    if privMode2DIsOn then beep : exit sub
    privMode2DIsOn=true
    ' make any 2D output as overlay over 3D primitives
    glDisable(GL_DEPTH_TEST)
    ' disable lighting calculations for 2D
    glDisable(GL_LIGHTING)
    
    if privBlendIsOn=false then
      glEnable(GL_BLEND)
    endif  
    ' active mode is modelview
    glPushMatrix()
      glLoadIdentity()
      ' switch to projection mode
      glMatrixMode(GL_PROJECTION)
      glPushMatrix()
        glLoadIdentity()
        glOrtho(0.0f, canvasWidth(), canvasHeight(), 0.0f, -1.0f, 1.0f)
        ' back to active modelview mode
        glMatrixMode(GL_MODELVIEW)
            
      
  elseif privMode2DIsOn then
        ' restore modelview matrix
        glPopMatrix()
      ' switch temporare to projection mde  
      glMatrixMode(GL_PROJECTION)
    ' restore projection matrix  
    glPopMatrix()
    ' enable last active modelview mode
    glMatrixMode(GL_MODELVIEW)
    
    glEnable(GL_LIGHTING)
    glEnable(GL_DEPTH_TEST)
    if privBlendIsOn=false then
      glDisable(GL_BLEND)
    endif      
    privMode2DIsOn=false
  end if 
end sub

' return frame rate (frames per second calculated by calling Advance) 
function glRenderer . FPS as integer
  return privFPS
end function
' return frame counter (invremented by calling Advance) 
function glRenderer . Frame as integer
  return privFrame
end function
' return current Timescale (1.0/FPS calculated by calling Advance)
function glRenderer . TimeScale as double
  if privScale<=0 then return 1.0/60.0
  return privScale
end function
' return runtime (calculated by calling Advance)
function glRenderer . Runtime as double
  return privNow - privStart
end function

' show render canvas on screen
sub glRenderer.Show
  flip
end sub

' ! IMPORTANT ! must be called every frame (updates the internal state of the glRenderer)
sub glRenderer.Advance(byval bSleep as boolean)
  privFrame+=1
  if (privFrame mod 25)=0 then
    privNow = Timer() : privFPS=25/(privNow-privLast) : privLast=privNow
    ' 80% from old timeScale and 20% from new timeScale
    privScale = privScale*0.8 + 1.0f/privFps*0.2
  end if  
  ' don't use all CPU cycles
  if bSleep then sleep 1,1
end sub


sub glRenderer . Text(byval x as integer, byval y as integer, byref txt as string, _
                    byval r as single , byval g as single , byval b   as single)
  var Mode2DWasOn = privMode2DIsOn
  var nChars = len(txt)
  var TextWidth = nChars*8
  if nChars<1 then exit sub
  ' complete outside top
  if y < -15 then exit sub
  ' complete outside bottom
  if y >= canvasHeight() then exit sub
  ' complete outside left
  if x+TextWidth < 0 then exit sub
  ' complete outside right
  if x >= canvasWidth() then exit sub
  
  ' last check
  if x+TextWidth >= canvasWidth() then
    ' modify the string length
    TextWidth = (canvasWidth()-1)-x
    nChars = (TextWidth+7)\8
  end if
  
  ' enable 2D mode ?
  if Mode2DWasOn=false then Mode2D(true)

  glColor4f(r,g,b,1)
  
  glEnable(GL_TEXTURE_2D)
  glBindTexture(GL_TEXTURE_2D,privFontTexture)
    
  glListBase(privCharBase)
  
  glPushMatrix()
    glTranslatef(x,y,0)
    glCallLists(nChars,GL_UNSIGNED_BYTE,strptr(txt))
  glPopMatrix()

  glListBase(0)
  
  glBindTexture(GL_TEXTURE_2D,0)
  glDisable(GL_TEXTURE_2D)
    

  ' was 2D mode of before ? 
  if Mode2DWasOn=false then Mode2D(false)
end sub


sub glRenderer.cls(byval bColorBuffer   as boolean, _
                 byval bDepthBuffer   as boolean, _
                 byval bStencilBuffer as boolean)
  dim as GLenum what
  if bColorBuffer   then what or= GL_COLOR_BUFFER_BIT
  if bDepthBuffer   then what or= GL_DEPTH_BUFFER_BIT
  if bStencilBuffer then what or= GL_STENCIL_BUFFER_BIT
  if what=0 then exit sub
  glClear(what)
end sub

sub glRenderer.GroundShadow(byval bOnOff as boolean)
  if bOnOff then
    if privShadowIsOn then exit sub
    privShadowIsOn=true
    glDisable(GL_DEPTH_TEST)
    glDisable(GL_LIGHTING)
    if privBlendIsOn=false then
      glEnable(GL_BLEND)
    end if  
    glColor4fv(@privShadowColor(0))
    glPushMatrix()
    glMultMatrixf(@privShadowMatrix(0,0))
  elseif privShadowIsOn then
    privShadowIsOn=false
    glPopMatrix()
    if privBlendIsOn=false then
      glDisable(GL_BLEND)
    end if 
    glEnable(GL_DEPTH_TEST)
    glEnable(GL_LIGHTING)
  end if 
end sub  


' ########################
' # class neAnimatedBody #
' ########################
sub glRenderer.Render(byval animatedBody as neAnimatedBody ptr)
  if animatedBody = NULL then exit sub
  
  if AnimatedBodyGetGeometryCount(animatedBody)<1 then exit sub
  
  if AnimatedBodyGetActive(animatedBody) then
    privRender(animatedBody,.2,.2,.8)
  else   
    privRender(animatedBody,.8,.2,.8)
  end if  
end sub

' #####################
' # class neRigidBody #
' #####################
sub glRenderer.Render(byval rigidBody as neRigidBody ptr)
  Render(rigidBody,false)
end sub  
sub glRenderer.Render(byval rigidBody as neRigidBody ptr, _
                      byval bDrawSensors as boolean)
  if rigidBody = NULL then exit sub
  if RigidBodyGetGeometryCount(rigidBody)<1 then exit sub
  
  if RigidBodyGetActive(rigidBody) then
    if RigidBodyIsIdle(rigidBody) then
      privRender(rigidBody, .8,.2,.2, bDrawSensors)
    else
      privRender(rigidBody, .2,.8,.2, bDrawSensors)
    end if  
  else
    privRender(rigidBody, .8,.2,.8, bDrawSensors)
  end if  
end sub

' ########################
' # class neTriangleMesh #
' ########################
sub glRenderer . Render(byval mesh as neTriangleMesh ptr)
  if mesh = NULL then exit sub
  if mesh->List = 0 then SetMesh(mesh)
  if mesh->List then glCallList(mesh->List)
end sub

sub glRenderer . SetMesh(byval mesh as neTriangleMesh ptr)
  if mesh               = NULL then exit sub
  if mesh->vertices     = NULL then exit sub
  if mesh->triangles    = NULL then exit sub
  if mesh->vertexCount  < 3    then exit sub
  if mesh->triangleCount< 1    then exit sub
  
  if mesh->List then glDeleteLists(mesh->List,1) : mesh->List=0
  mesh->List = glGenLists(1)
  glNewList(mesh->list, GL_COMPILE)
    glBegin (GL_TRIANGLES)
      glColor3f(.75, .5, .25)
      for triangleIndex as integer = 0 to mesh->triangleCount-1
        var vertA = mesh->vertices[mesh->triangles[triangleIndex].indices(0)]
        var vertB = mesh->vertices[mesh->triangles[triangleIndex].indices(1)]
        var vertC = mesh->vertices[mesh->triangles[triangleIndex].indices(2)]
        var triNormal = V3NormalizedCross(V3Sub(vertB,vertA),V3Sub(vertC,vertA))
        glNormal3fv(triNormal)
        glVertex3fv(vertA)
        glVertex3fv(vertB)
        glVertex3fv(vertC)
      next  
    glEnd()
  glEndList() ' after this call the complete triangle mesh are in GPU memory !
end sub

' #################
' # class neJoint #
' #################
sub glRenderer.Render(byval j as neJoint ptr)
  dim as neV3 posA,posB 
  if j=NULL then return
  if JointGetType(j)<>NE_HINGE then
    var rba = JointGetRigidBodyA(j)
    var rbb = JointGetRigidBodyB(j)
    var abb = JointGetAnimatedBodyB(j)
    if rba<>NULL andalso rbb<>NULL then
      posA = RigidBodyGetPositionV3(rba)
      posB = RigidBodyGetPositionV3(rbb)
    elseif rba<>NULL andalso abb<>NULL then
      posA = RigidBodyGetPositionV3(rba)
      posB = AnimatedBodyGetPositionV3(abb)
    elseif rba<>NULL then
      posA = RigidBodyGetPositionV3(rba)
      var t=JointGetFrameB(j)
      posB = t.pos
    else
      exit sub
    end if
  else
    posA=JointGetFrameA(j).pos
    posB=JointGetFrameB(j).pos
  end if 
  
  if privShadowIsOn=false then
    glColor4f(.8,.8,.2,privAlpha)
    glDisable(GL_LIGHTING)
    'glDisable(GL_DEPTH_TEST)
  end if
  
  glBegin(GL_POINTS)
    glVertex3fv(posA)
    glVertex3fv(posB)
  glEnd()
   
  if privShadowIsOn=false then
    glColor4f(.8,.2,.8,privAlpha)
  end if
  glBegin(GL_LINES)
    glVertex3fv(posA)
    glVertex3fv(posB)
  glEnd()
  
  if privShadowIsOn=false then
    glEnable(GL_LIGHTING)
    'glEnable(GL_DEPTH_TEST)
  end if  

  ' to do:
  ' draw the limits of a joint (if enabled)
  ' draw the motor of a hinge joint (if enabled)
  ' ...
  select case as const JointGetType(j)
  case NE_BALLSOCKET
  case NE_HINGE
  case NE_SLIDER
  end select

end sub


' ##########################
' # from here it's private #
' ##########################

sub glRenderer . privSetLight()
  dim as single ambient(...) => {0.1,0.1,0.1,1.0}
  dim as single diffuse(...) => {0.8,0.8,0.8,1.0}
  dim as single specular(...) => {.8,.8,.2,1}
  dim as single shininess = 64
  'privAutoNormaIsOn=true
  'glEnable(GL_AUTO_NORMAL)
  
  'glEnable(GL_COLOR_MATERIAL)
  'glMaterialfv(GL_FRONT, GL_AMBIENT,   @ambient(0))
  'glMaterialfv(GL_FRONT, GL_DIFFUSE,   @diffuse(0))
  'glMaterialfv(GL_FRONT, GL_SPECULAR,  @specular(0))
  'glMaterialfv(GL_FRONT, GL_SHININESS, @shininess)
  
  privLightPosition(0) = 0
  privLightPosition(1) = 100
  privLightPosition(2) =-50
  privLightPosition(3) = 1.0
  
  glEnable (GL_LIGHT0)
  'glLightfv(GL_LIGHT0,GL_AMBIENT ,@ambient(0))
  'glLightfv(GL_LIGHT0,GL_DIFFUSE ,@diffuse(0))
  'glLightfv(GL_LIGHT0,GL_SPECULAR,@specular(0))
  
  glLightfv(GL_LIGHT0,GL_POSITION,@privLightPosition(0))
  
  privShadowMatrix(0,0)=privLightPosition(1) ' Y
  privShadowMatrix(0,1)=0.0f 
  privShadowMatrix(0,2)=0.0f
  privShadowMatrix(0,3)=0.0f

  privShadowMatrix(1,0)=privLightPosition(0) ' X
  privShadowMatrix(1,1)=1.0f
  privShadowMatrix(1,2)=-privLightPosition(2) ' Z
  privShadowMatrix(1,3)=1.0 
  
  privShadowMatrix(2,0)=0.0f
  privShadowMatrix(2,1)=0.0f
  privShadowMatrix(2,2)=privLightPosition(1) ' Y
  privShadowMatrix(2,3)=0.0f
  
  privShadowMatrix(3,0)=0.0f
  privShadowMatrix(3,1)=0.0f
  privShadowMatrix(3,2)=0.0f
  privShadowMatrix(3,3)=privLightPosition(1) ' Y

  privShadowColor(0)=0.0f
  privShadowColor(1)=0.0f
  privShadowColor(2)=0.0f
  privShadowColor(3)=0.3f

end sub  

sub glRenderer.privPerspective(byval fov as single, byval ratio as single, byval zNear as single, byval zFar as single)
  if fov< 15.0f then fov = 15.0f
  if fov>120.0f then fov = 120.0f
  if ratio<0.1f then ratio = canvasWidth()/canvasHeight()
  if zNear<0.1f then zNear=0.1f
  if zFar<=zNear then zFar=zNear+1024.0f
  dim as single fH = neAtn( fov / 360.0 * NE_PI ) * zNear
  dim as single fW = fH * ratio
  glFrustum(-fW,fW, -fH,fH, zNear,zFar)
end sub

sub glRenderer.privRender(byval animatedBody as neAnimatedBody ptr, _
                        byval r as single, _
                        byval g as single,_
                        byval b as single)
  glPushMatrix()
    'glLoadIdentity()
    AnimatedBodyGetOpenGLMatrix(animatedBody,privMatrix)
    glMultMatrixF(privMatrix)
    
    if privShadowIsOn=false then 
      glColor4f(r,g,b,privAlpha)
    end if  
      
    AnimatedBodyBeginIterateGeometry(animatedBody)
    var geo = AnimatedBodyGetNextGeometry(animatedBody)
    while geo
      privRender(geo) 
      geo = AnimatedBodyGetNextGeometry(animatedBody)
    wend
  glPopMatrix()
end sub

sub glRenderer.privRender(byval rigidBody as neRigidBody ptr, _
                        byval r as single, _
                        byval g as single, _
                        byval b as single, _
                        byval bDrawSensors as boolean)
  glPushMatrix()
    RigidBodyGetOpenGLMatrix(rigidBody, privMatrix)
    glMultMatrixF(privMatrix)
    
    if privShadowIsOn=false then
      glColor4f(r,g,b,privAlpha)
    end if  
    
    RigidBodyBeginIterateGeometry(rigidBody)
    var geo = RigidBodyGetNextGeometry(rigidBody)
    while geo
      privRender(geo)
      geo = RigidBodyGetNextGeometry(rigidBody)
    wend
    
    if bDrawSensors then
      RigidBodyBeginIterateSensor(rigidBody)
      var sen = RigidBodyGetNextSensor(rigidBody)
      while sen
        privRender(sen) 
        sen = RigidBodyGetNextSensor(rigidBody)
      wend  
    end if  
    
  glPopMatrix()
end sub

sub glRenderer.privRender(byval geo as neGeometry ptr)
  dim as single fDiameter=any,fWidth=any,fHeight=any,fDepth
  
  glPushMatrix()
  
    GeometryGetOpenGLMatrix(geo, privMatrix)
    glMultMatrixf(privMatrix)

    if GeometryGetBoxSize(geo, fWidth, fHeight, fDepth) then
      var bScale = false
      if fWidth<>1.0f orelse fHeight<>1.0f orelse fDepth<>1.0f then bScale=true
      ' push and pop and scale only if really needed !
      if bScale then 
        glPushMatrix() 
        glScalef(fWidth, fHeight, fDepth)
      end if  
      ' render a box
      glCallList(privBoxList)
      ' restore the matix
      if bScale then 
        glPopMatrix()
      end if  
      
    elseif GeometryGetSphereDiameter(geo,fDiameter)  then  
      var bScale = false
      if fDiameter<>1.0f then bScale=true
      if bScale then 
        glPushMatrix()
        glScalef(fDiameter,fDiameter,fDiameter)
      end if  
      ' render a sphere
      glCallList(privSphereList)
      
      if bScale then
        glPopMatrix()
      end if  
      
    elseif GeometryGetCapsule(geo,fDiameter,fHeight) then  
      dim as single fCapY=fDiameter*0.25f + fHeight/2.0f
      ' render a capsule top
      glPushMatrix() 
        glTranslatef(0,fCapY,0)
        glScalef(fDiameter, fDiameter, fDiameter)
        glCallList(privCapsuleTop)
      glPopMatrix()
     
      ' render the capsule body (cylinder)
      glPushMatrix()
        glScalef(fDiameter, fHeight, fDiameter)
        glCallList(privCapsuleBody) 
      glPopMatrix()
      
      ' render a capsule top
      glPushMatrix()
        glTranslatef(0,-fCapY,0)
        glScalef(fDiameter, fDiameter, fDiameter)
        glCallList(privCapsuleBottom)
      glPopMatrix()
    end if
    
  glPopMatrix()  
end sub

sub glRenderer.privRender(byval sen as neSensor ptr)
  dim as neV3 p=any,d=any
  if sen=NULL then return
  ' draw a line
  p = SensorGetLinePositionV3(sen)
  d = SensorGetLineVectorV3(sen)
  if privShadowIsOn=false then
    glColor4f(.8,.8,.2,privAlpha)
    glDisable(GL_LIGHTING)
  end if  
  glLineWidth(3)
  glBegin(GL_LINES)
    glVertex3fv(p)
    p.x+=d.x : p.y+=d.y : p.z+=d.z
    glVertex3fv(p)
  glEnd()
  glLineWidth(1)
  if privShadowIsOn=false then
    glEnable(GL_LIGHTING)
  end if
end sub

'  ###############################
' # create a unit 1 x 1 x 1 box #
'###############################  
function glRenderer.privCreateCube() as GLuint
  var ret = glGenLists(1)
  glNewList(ret,GL_COMPILE)
  glBegin(GL_QUADS)
    glNormal3f( 0.0f, 0.0f, 1.0f) ' front
    glVertex3f(-0.5f, 0.5f, 0.5f)
    glVertex3f(-0.5f,-0.5f, 0.5f)
    glVertex3f( 0.5f,-0.5f, 0.5f)
    glVertex3f( 0.5f, 0.5f, 0.5f)
    
    glNormal3f( 0.0f, 0.0f,-1.0f) ' back
    glVertex3f( 0.5f, 0.5f,-0.5f)
    glVertex3f( 0.5f,-0.5f,-0.5f)
    glVertex3f(-0.5f,-0.5f,-0.5f)
    glVertex3f(-0.5f, 0.5f,-0.5f)
    
    glNormal3f( 1.0f, 0.0f, 0.0f) ' right
    glVertex3f( 0.5f, 0.5f, 0.5f)
    glVertex3f( 0.5f,-0.5f, 0.5f)
    glVertex3f( 0.5f,-0.5f,-0.5f)
    glVertex3f( 0.5f, 0.5f,-0.5f)
    
    glNormal3f(-1.0f, 0.0f, 0.0f) ' left
    glVertex3f(-0.5f, 0.5f,-0.5f)
    glVertex3f(-0.5f,-0.5f,-0.5f)
    glVertex3f(-0.5f,-0.5f, 0.5f)
    glVertex3f(-0.5f, 0.5f, 0.5f)

    glNormal3f( 0.0f, 1.0f, 0.0f) ' top
    glVertex3f(-0.5f, 0.5f,-0.5f)
    glVertex3f(-0.5f, 0.5f, 0.5f)
    glVertex3f( 0.5f, 0.5f, 0.5f)
    glVertex3f( 0.5f, 0.5f,-0.5f)
    
    glNormal3f( 0.0f,-1.0f, 0.0f) ' bottom
    glVertex3f(-0.5f,-0.5f, 0.5f)
    glVertex3f(-0.5f,-0.5f,-0.5f)
    glVertex3f( 0.5f,-0.5f,-0.5f)
    glVertex3f( 0.5f,-0.5f, 0.5f)
  glEnd()
  glEndList()
  return ret
end function 

'  ##################################
' # create a unit 1 x 1 x 1 sphere # (radius=0.5)
'################################## 
function glRenderer.privCreateSphere() as GLuint
  const as integer SEGMENTS = 16
  const as single  US       = (NE_PI*2) / SEGMENTS
  const as single  VS       =  NE_PI    / SEGMENTS
  const as integer NPOINTS  = (SEGMENTS + 1) * (SEGMENTS + 1)      
  dim as single P(NPOINTS*3-1)  
  dim as GLuint list
  dim as integer pc,yc,xc',p0,p1,p2,p3
  dim as single VW,UW,UR,YP,VR
  'dim as neV3 a,b,c,d,n
  
  For yc = 0 To SEGMENTS
    UR = neSin(VW) * 0.5f
    YP = neCos(VW) * 0.5f
    VR = neSin(VW) * 0.5f
    VW+= VS
    UW = 0
    For xc = 0 To SEGMENTS
      P(PC+0)=neSin(UW)*UR
      P(PC+1)=YP
      P(PC+2)=neCos(UW)*VR ' !!! neCos(NE_PI+UW)
      PC+=3
      UW+=US
    Next
  Next

  list = glGenLists(1)
  glNewList(list,GL_COMPILE)
    glBegin(GL_TRIANGLES)
      For yc= 0 To SEGMENTS - 1
        For xc= 0 To SEGMENTS - 1
          var P0 = ((yc + 0) * (SEGMENTS + 1) + (xc + 0))*3
          var P1 = ((yc + 0) * (SEGMENTS + 1) + (xc + 1))*3
          var P2 = ((yc + 1) * (SEGMENTS + 1) + (xc + 1))*3
          var P3 = ((yc + 1) * (SEGMENTS + 1) + (xc + 0))*3
          var A=neV3(P(P0),P(P0+1),P(P0+2))
          var B=neV3(P(P1),P(P1+1),P(P1+2))
          var C=neV3(P(P2),P(P2+1),P(P2+2))
          var D=neV3(P(P3),P(P3+1),P(P3+2))
          var N=V3Normalized(A) : glNormal3fv(N) : glVertex3fv(A)
              N=V3Normalized(B) : glNormal3fv(N) : glVertex3fv(B)
              N=V3Normalized(D) : glNormal3fv(N) : glVertex3fv(D)
              
              N=V3Normalized(B) : glNormal3fv(N) : glVertex3fv(B)
              N=V3Normalized(C) : glNormal3fv(N) : glVertex3fv(C)
              N=V3Normalized(D) : glNormal3fv(N) : glVertex3fv(D)
        Next
      Next
    glEnd()
  glEndList()
  return list
end function                

'  ########################
' # create a 1x2 capsule #
'######################## 
sub glRenderer.privCreateCapsule(byref top    as GLuint, _
                           byref body   as GLuint, _
                           byref bottom as GLuint)
  dim as single l,r
  dim as integer i,j,n,ret
  dim as single tmp,a,ca,sa
  dim as single nx ,ny ,nz ,start_nx ,start_ny
  dim as single nx2,ny2,nz2,start_nx2,start_ny2
  
  if top<>0 orelse body<>0 orelse bottom<>0 then exit sub
  
  ' number of sides to the cylinder (divisible by 4):
  n = 40
  l = 0.5
  r = 0.5
  a  = (NE_PI*2.0)/n
  sa = neSin(a)
  ca = neCos(a)

  ' draw capsule body height 2 (cylinder)
  nx=1:ny=0:nz=0 ' normal vector
  body = glGenLists(1)
  glNewList(body, GL_COMPILE)
    glBegin (GL_TRIANGLE_STRIP)
    for i=0 to n 
      glNormal3f(nx,0,nz) : glVertex3f(nx*r,-.5f,nz*r) 
      glNormal3f(nx,0,nz) : glVertex3f(nx*r, .5f,nz*r)
      ' rotate nx,nz
      tmp = ca*nx - sa*nz
      nz  = sa*nx + ca*nz
      nx  = tmp
    next
    glEnd()
  glEndList() 
 
  ' draw capsule top cap
  top = glGenLists(1)
  glNewList (top, GL_COMPILE)
    start_nx = 0
    start_ny = 1
    for j=0 to (n/4)-1
      ' get start_n2 = rotated start_n
      start_nx2 =  ca*start_nx + sa*start_ny
      start_ny2 = -sa*start_nx + ca*start_ny
      ' get n=start_n and n2=start_n2
      nx = start_nx : ny  = start_ny : nz = 0
      nx2= start_nx2: ny2 = start_ny2: nz2= 0
      glBegin (GL_TRIANGLE_STRIP)
        for i= 0 to n
          glNormal3f(ny ,nx ,nz ) : glVertex3f(ny *r,nx *r-0.25,nz *r)
          glNormal3f(ny2,nx2,nz2) : glVertex3f(ny2*r,nx2*r-0.25,nz2*r)
          ' rotate n,n2
          tmp = ca*ny - sa*nz
          nz  = sa*ny + ca*nz
          ny  = tmp
          tmp = ca*ny2 - sa*nz2
          nz2 = sa*ny2 + ca*nz2
          ny2 = tmp
        next
      glEnd()
      start_nx = start_nx2
      start_ny = start_ny2
    next   
  glEndList() 

  bottom=glGenLists(1)
  glNewList (bottom, GL_COMPILE)
    ' draw capsule bottom cap
    start_nx = 0
    start_ny = 1
    for j=0 to (n/4)-1
      ' get start_n2 = rotated start_n
      start_nx2 = ca*start_nx - sa*start_ny
      start_ny2 = sa*start_nx + ca*start_ny
      ' get n=start_n and n2=start_n2
      nx = start_nx: ny = start_ny: nz = 0
      nx2 = start_nx2: ny2 = start_ny2: nz2 = 0
      glBegin (GL_TRIANGLE_STRIP)
        for i=0 to n
          glNormal3f (ny2,nx2,nz2) : glVertex3f (ny2*r,nx2*r+0.25,nz2*r)
          glNormal3f (ny ,nx ,nz ) : glVertex3f (ny *r,nx *r+0.25,nz *r)
      
          ' rotate n,n2
          tmp = ca*ny - sa*nz
          nz  = sa*ny + ca*nz
          ny  = tmp
          tmp = ca*ny2 - sa*nz2
          nz2 = sa*ny2 + ca*nz2
          ny2 = tmp
        next
      glEnd()
      start_nx = start_nx2
      start_ny = start_ny2
    next
  glEndList() 
end sub
