#pragma once
#include "tokamak-c.bi"

' This is an example to show you how easy you can extends a neClass to your own needs.

' Remember the neTerrain class is a static collision shape bulid from an neTriangleMesh.
' In this example I extend the neTriangleMesh to build a random neSandbox class :-)

' I implement only a neSandbox constructor and use AddVertex and addTriangle from neTriangleMesh class !

' The physics renderer I wrote can render neTriangleMeshes 
' so it can render any extended neTriangleMesh class also.

type neSandbox extends neTriangleMesh
  declare constructor (byval xzSize      as integer = 64, _
                       byval randomRange as single  = 10.0f, _
                       byval smoothSteps as integer = -1)
  as neV3 minBound
  as neV3 maxBound                       
end type

' Create a random sandbox as neTriangleMesh
constructor neSandbox(byval xzSize      as integer, _
                      byval randomRange as single , _
                      byval smoothsteps as integer)
  dim as integer mask
  if xzSize<8 then xzSize=8
  mask = xzSize-1
  minBound=neV3( xzSize, randomRange, xzSize)
  maxBound=neV3(-xzSize,-randomRange,-xzSize)

  if smoothsteps<0 then smoothsteps=1+int(randomRange)
    
  ' a simple grid of vectors with random y values
  dim as neV3 ptr v = callocate(xzSize*xzSize*sizeof(neV3))
  ' Grid(x,y) looks more as a "normal" 2d array :-)
  #define Grid(col,row) v[(row)*xzSize + (col)]
  for z as integer = 0 to mask
    for x as integer = 0 to mask
      Grid(x,z).x=-xzSize/2 + x
      Grid(x,z).y= (rnd-rnd)*randomRange
      Grid(x,z).z=-xzSize/2 + z
      if smoothsteps<1 then
        if Grid(x,z).x > maxBound.x then maxBound.x=Grid(x,z).x
        if Grid(x,z).x < minBound.x then minBound.x=Grid(x,z).x
        if Grid(x,z).y > maxBound.y then maxBound.y=Grid(x,z).y
        if Grid(x,z).y < minBound.y then minBound.y=Grid(x,z).y
        if Grid(x,z).z > maxBound.z then maxBound.z=Grid(x,z).z
        if Grid(x,z).z < minBound.z then minBound.z=Grid(x,z).z
      end if  
    next
  next

  ' smooth vertices
  if smoothsteps>0 then
    for i as integer = 1 to smoothsteps
      for z as integer = 0 to mask
        dim as integer zp = (z+1) : if zp=xzSize then zp=0
        for x as integer = 0 to mask
          dim as integer xp = (x+1): if xp=xzSize then xp=0
          Grid(x,z).y=(Grid(x ,z ).y + _
                       Grid(xp,z ).y + _
                       Grid(x ,zp).y + _
                       Grid(xp,zp).y)*0.25f
          if i=smoothsteps then
            if Grid(x,z).x > maxBound.x then maxBound.x=Grid(x,z).x
            if Grid(x,z).x < minBound.x then minBound.x=Grid(x,z).x
            if Grid(x,z).y > maxBound.y then maxBound.y=Grid(x,z).y
            if Grid(x,z).y < minBound.y then minBound.y=Grid(x,z).y
            if Grid(x,z).z > maxBound.z then maxBound.z=Grid(x,z).z
            if Grid(x,z).z < minBound.z then minBound.z=Grid(x,z).z
          end if  
        next
      next
    next
  end if  
  ' create a border for the sandbox
  var d=V3Sub(maxBound,minBound)
  for b as integer = 0 to mask
    Grid(0,b).y=-d.y
    Grid(b,0).y=-d.y
    Grid(mask,b).y=-d.y
    Grid(b,mask).y=-d.y
  next
  for b as integer = 1 to mask-1
    for r as integer=0 to 1
      Grid(1+r,b).y=d.y
      Grid(b,1+r).y=d.y
      Grid(mask-(1+r),b).y=d.y
      Grid(b,mask-(1+r)).y=d.y
    next  
  next  

  ' build triangle mesh from grid
  for z as integer = 0 to mask-1
    for x as integer = 0 to mask-1
      ' pick 4 vertices from grid
      var A = AddVertex(Grid(x    ,z    ))
      var B = AddVertex(Grid(x    ,(z+1)))
      var C = AddVertex(Grid((x+1),(z+1)))
      var D = AddVertex(Grid((x+1),z    ))
      
      var Ay = Grid(x    ,z    ).y
      var By = Grid(x    ,(z+1)).y
      var Cy = Grid((x+1),(z+1)).y
      var Dy = Grid((x+1),z    ).y
      ' choose a axis (A-C or B-D) to split the quad in two triangles
      ' (of course I could swap A,B,C,D but I will show you how it works)
      if neAbs(Ay - Cy) >= neAbs(By-Dy) then
        ' A----D
        ' |\(2)|
        ' | \  |
        ' |  \ |
        ' |(1)\|
        ' B----C
        ' add triangle 1 from vertices A,B,C (counter clockwise)
        AddTriangle(A,B,C)
        ' add triangle 2 from vertices A,C,D (counter clockwise)
        AddTriangle(A,C,D)
      else
        ' A----D
        ' |(1)/|
        ' |  / |
        ' | /  |
        ' |/(2)|
        ' B----C
        ' add triangle 1 from vertices A,B,D (counter clockwise)
        AddTriangle(A,B,D)
        ' add triangle 2 from vertices D,B,C (counter clockwise)
        AddTriangle(D,B,C)
      end if  
    next
    sleep 1,1
  next
  #undef Grid
  delete v
end constructor
