#pragma once

' FreeBASIC: 
' optional move/copy *.bi files to in FreeBASIC include folder

' Linux: 
' move/copy (as sudo) tokamak-c-32.so or tokamak-c-64.so to your library folder

#ifndef __FB_64BIT__
 #inclib "tokamak-1.2-c-32"  
#else
 #inclib "tokamak-1.2-c-64"  
#endif


#include "crt.bi"

#ifndef NULL
#define NULL cptr(any ptr,0)
#endif


' I use the float C runtime functions
#define neSqr(x)     sqrtf((x))
#define neCos(x)      cosf((x))
#define neACos(x)    acosf((x))
#define neSin(x)      sinf((x))
#define neASin(x)    asinf((x))
#define neAtn(x)      tanf((x))
#define neATan(x)    atanf((x))
#define neATan2(y,x) atan2f((y),(x))

const as single NE_PI   = 3.141592654f
const as single NE_ZERO = 1.0e-6f

const as single DEG2RAD = NE_PI/180.0f
const as single RAD2DEG = 180.0f/NE_PI

#define neAbs(x)            iif( (x)<0, -(x), (x) )
#define neIsConsiderZero(f) iif( neAbs(f)<NE_ZERO, true, false )
#define neIsEqual(x,y)      iif( (2.0f*neAbs((x)-(y))/((x)+(y))) < NE_ZERO, true, false )

#define neRnd(fMin,fMax) (fMin + (fMax-fMin)*rnd())



type neV3 ' NOTE: 4*SizeOf(single)=16 bytes NOT 3*SizeOf(single)=12 bytes !
  declare constructor
  declare constructor(x as single,y as single,z as single)
  declare const operator cast as const single ptr
  declare const operator cast as string
  static as const neV3 Zero
  static as const neV3 One  
  static as const neV3 Right
  static as const neV3 Left
  static as const neV3 Up
  static as const neV3 Down
  static as const neV3 Forward
  static as const neV3 Backward  
  as single x=any,y=any,z=any, pad=any
end type
constructor neV3
end constructor
constructor neV3(a as single,b as single,c as single)
  x=a:y=b:z=c
end constructor
const operator neV3 . cast as const single ptr
  return @this.x
end operator
operator neV3 . cast as string
  return "[" & x & "," & y & "," & z & "]"
end operator
' ##############
' # class neM3 #
' ##############
type neM3 ' NOTE: 3*4*SizeOf(Single)=48 bytes NOT 3*3*SizeOf(Single)=36 bytes !
  declare constructor
  declare const operator cast as string
  as single m00=any,m01=any,m02=any, pad03=any
  as single m10=any,m11=any,m12=any, pad13=any
  as single m20=any,m21=any,m22=any, pad23=any
end type
constructor neM3
end constructor
operator neM3 . cast as string
  return "[row0 " & m00 & "," & m01 & "," & m02 & _
         " row1 " & m10 & "," & m11 & "," & m12 & _
         " row2 " & m20 & "," & m21 & "," & m22 & "]"
end operator
' ##############
' # class neT3 #
' ##############
type neT3 ' NOTE: 16+48=64 bytes NOT 12+36=48 bytes !
  declare constructor
  declare const operator cast as string
  as neM3 rot
  as neV3 pos
end type
constructor neT3
end constructor
operator neT3 . cast as string
  return "[row0 " & rot.m00 & "," & rot.m01 & "," & rot.m02 & _
         " row1 " & rot.m10 & "," & rot.m11 & "," & rot.m12 & _
         " row2 " & rot.m20 & "," & rot.m21 & "," & rot.m22 & _
         "  pos " & pos.x   & "," & pos.y   & "," & pos.z   &  "]"
end operator
' #############
' # class neQ #
' #############
type neQ
  as single x=any,y=any,z=any,w=any
end type

dim neV3.Zero     as const neV3 = neV3( 0f, 0f, 0f)
dim neV3.One      as const neV3 = neV3( 1f, 1f, 1f)

dim neV3.Left     as const neV3 = neV3(-1f, 0f, 0f)
dim neV3.Right    as const neV3 = neV3( 1f, 0f, 0f)

dim neV3.Down     as const neV3 = neV3( 0f,-1f, 0f)
dim neV3.Up       as const neV3 = neV3( 0f, 1f, 0f)

dim neV3.Forward  as const neV3 = neV3( 0f, 0f, 1f)
dim neV3.Backward as const neV3 = neV3( 0f, 0f,-1f)



' The default constructor of the physic simlulator used this low but memory friendly default values.
' var sim = CreateSimulator()
' if you need more objects you have to setup your own neSimulatorSizeInfo structure
' and used it as argument for the constructor of the simulator.
' NOTE: To make it more simple you can use CreateSimulatorEx() from thje helpers section also.
const as long DEFAULT_RIGIDBODIES_COUNT          = 50   ' (50)
const as long DEFAULT_ANIMATEDBODIES_COUNT       = 50   ' (50)
const as long DEFAULT_RIGIDPARTICLES_COUNT       = 50   ' (50)
const as long DEFAULT_GEOMETRIES_COUNT           = DEFAULT_RIGIDBODIES_COUNT _
                                                 + DEFAULT_ANIMATEDBODIES_COUNT  ' (100)
const as long DEFAULT_OVERLAPPED_PAIRS_COUNT     = (DEFAULT_GEOMETRIES_COUNT*(DEFAULT_GEOMETRIES_COUNT-1))\2 ' (1225)
const as long DEFAULT_CONTROLLERS_COUNT          = 50   ' (50)
const as long DEFAULT_SENSORS_COUNT              = 100  ' (100)
const as long DEFAULT_CONSTRAINTS_COUNT          = 100  ' (100)
const as long DEFAULT_CONTRAINT_SETS_COUNT       = 100  ' (100)
const as long DEFAULT_CONTRAINT_BUFFER_SIZE      = 2000 ' (2000)
const as long DEFAULT_TERRAIN_NODES_START_COUNT  = 200  ' (200)
const as long DEFAULT_TERRAIN_NODES_GROWBY_COUNT = -1

' The neSimulatorSizeInfo struct are optinal used to init the neSimulator.
' If you need more objects as defined as default you have to setup the 
' neSimulatorSizeInfo struct self and use it as argument for CreateSimulator().
' NOTE: To make it more simple you can use CreateSimulatorEx() from the helpers section also.
type neSimulatorSizeInfo
  declare constructor
  ' max number of rigid bodies in the simulation (default: 50)
  as long rigidBodiesCount 
  ' max number of staic bodies in the simulation (default: 50)
  as long animatedBodiesCount
  ' max number of rigid particles in the simulation (default: 50)
  as long rigidParticleCount 
  ' max number of controller instances in the simulation (default: 50)
  as long controllersCount 
  ' Number of possible overlapping pairs. (default: 1225)
  ' This has the maximum value of (n x (n - 1)) / 2, where n = rigidBodyCount + animatedBodyCount.
  ' But in practice it rarely reach that high. You can try to specify a smaller number to save memory.
  as long overlappedPairsCount
  ' max number of collision geometries in the simulator (default: 100)
  as long geometriesCount
  ' max number of joints in the simulation (default: 100)
  as long constraintsCount 
  ' max number of joint Sets in the simulation (default: 100)
  as long constraintSetsCount 
  ' max size of the buffer used to solve joint contraince (default: 2000)
  ' (looks like the default value is good for any number of joints)
  as long constraintBufferSize 
  ' max number of sensors in the simulation (default: 100)
  as long sensorsCount  
  ' max number of nodes use to store terrain triangles (default: 200)
  as long terrainNodesStartCount 
  ' Grow by this size if run out of nodes (default: -1)
  ' -1 the default value means: signify double the number of terrainNode, whenever the it reach full capacity.
  as long terrainNodesGrowByCount
end type

constructor neSimulatorSizeInfo
  ' The constructor initialises the members to the low but memory friendly default values.
  rigidBodiesCount        = DEFAULT_RIGIDBODIES_COUNT
  animatedBodiesCount     = DEFAULT_ANIMATEDBODIES_COUNT
  rigidParticleCount      = DEFAULT_RIGIDPARTICLES_COUNT
  controllersCount        = DEFAULT_CONTROLLERS_COUNT
  overlappedPairsCount    = DEFAULT_OVERLAPPED_PAIRS_COUNT
  geometriesCount         = DEFAULT_GEOMETRIES_COUNT
  constraintsCount        = DEFAULT_CONSTRAINTS_COUNT
  constraintSetsCount     = DEFAULT_CONTRAINT_SETS_COUNT
  constraintBufferSize    = DEFAULT_CONTRAINT_BUFFER_SIZE
  sensorsCount            = DEFAULT_SENSORS_COUNT
  terrainNodesStartCount  = DEFAULT_TERRAIN_NODES_START_COUNT
  terrainNodesGrowByCount = DEFAULT_TERRAIN_NODES_GROWBY_COUNT
end constructor

#define neClass(_name_) type _name_ : as integer dummy : end type

' class neSimulator solves forces on bodies and handle collision between collision shapes
neClass(neSimulator)

' Using the class neCollisionTable, the application can specify the response 
' whenever two bodies collide in the simulation.
' Each body in the simulation has a collision ID, which defaults to 0.
' A collision table contains a neResponseBitFlag for each pair of collision IDs.
' A neResponseBitFlag equal to RESPONSE_IGNORE means the collision will be ignored
' no collision impulse or collision callback will be performed.
' The application can retrieve the neCollisionTable by calling neSimulatorGetCollisionTable() function.
neClass(neCollisionTable)

' Class neGeometry describes a collision shape and used by neAnimatedBody and neRigidBody
neClass(neGeometry)

' Class neAnimatedBody has infinty mass and one or more collision shapes (neGeometry) like a building ...
' and will never moved or rotated by the simulator (after a collision with a neRigidBody for example)
' "animated" means it can only animated (translation or rotation) manual by you !
' neAnimatedBody are fixed in space during a simulation step and can't collide with other neAnimatedBodies
' NOTE: you can define a geometry as breakable !
' if a defined limit of forces act on it, it will be split in parts (converted to neRigidBodies).
neClass(neAnimatedBody)

' Class neRigidBody has a mass and optional one or more collision shapes (neGeometry)
' The position and rotation of a RigidBody are calculated by the law's of physics.
' A rigid body with one or more collision shapes interact with all other rigid- or animated bodies.
' (one exeption if a neRigidBody are created as a particle it will never collide with other particles)
neClass(neRigidBody)

' Class neRigidBodyController are used inside an optional user defined neRigidBodyCallback
neClass(neRigidBodyController)

' Class neJoint describes the constrain of agility of an neRigidBody or neAnimatedBody. 
' neJoint can connect a neRigidBody with an fixed point in world space.
' neJoint can connect a neRigidBody with an other neRigidBody.
' neJoint can connect a neRigidBody with an neAnimatedBody.
' NOTE: the rotation axis of hinge joint can act as an motor also!
neClass(neJoint)

' Class neJointController are used inside a optional user defined joint callback
neClass(neJointController)

' Class neSensor is a 2D line with a position and direction.
' (Internal the length and normalized direction are calcualted)

' When the line representing the neSensor intersects with any of the collision objects
' the intersection information can be retrieved and used by the application.

' A simple position and direction that presents a sensor 
' sounds not really spectacular but believe me it's a really powerful tool 
' that can control a complete game logic with few lines of code.

' Simple example:
' You have a bunch of buildings with a lot of rooms in your 3D game world 
' if you put a line sensor in front and/or behind all the doors 
' every time a "object" intersects with this line sensor ...
' You can open or close the door if the "object" is allowed to enter or leave.
' You can count number of "objects" enters the building or a room !
' You can count number of "objects" leavs the building or a room !
' You can count number of "objects" are currently in the building and in wich room.
' ...

' Complex example:
' You have cars or any kind of vehicles with wheels
' add for every wheel a line sensor that points down to the ground.
' depending of the penetrations depth of the line (that represents a wheel) and the ground
' you can use it to calculate the behavior of the wheel and the vehicles rigidbody that are the owner of the wheel.

' While one sensor line represent the wheel a second sensor can act as a spring/damper.
' The start of the spring/damper line is connected to the vehicle and the end of the damper
' are connected to the line wich represents the wheel connected to the wheeled vehicle.

' In short words with some "simple logic math" you can control a complete game.
' With complex physics math you can create any machine you can imagine.
neClass(neSensor)

' neTriangle is a single item of an neTriangleMesh
type neTriangle
  as long    indices(2)
  as long    materialID
  as ulong   flag
  as any ptr userData
end type

' neTriangleMesh is a collection of neTriangles and are used to describe a static terrain.
' The terrain are internaly stored in a tree structure the treenodes are really fast 
' to detect collision with neRigidBodies and neSensors.
' This makes it perfect to describe many static parts of your simulated world. 
' (ground, mountains, buildings, streets ...)
type neTriangleMesh
  ' add a vertex and return the vertexIndex
  declare function AddVertex(byval x as single, _
                             byval y as single, _
                             byval z as single, _
                             byval bOnce as boolean=true) as integer

  ' add a vertex and return the vertexIndex
  declare function AddVertex(byref vertex as const neV3, _
                             byval bOnce as boolean=true) as integer
  ' add a triangle from three vertexIndices added before
  declare function AddTriangle(byval vetexIndex0 as integer, _
                               byval vetexIndex1 as integer, _
                               byval vetexIndex2 as integer, _
                               byval materialID  as integer=0, _
                               byval bOnce as boolean=false) as integer
  ' add a triangle from three vertices
  declare function AddTriangle(byref vertex0 as const neV3, _
                               byref vertex1 as const neV3, _
                               byref vertex2 as const neV3, _
                               byval materialID as integer=0, _
                               byval bOnce as boolean=false) as integer                               
  as neV3 ptr       vertices
  as long           vertexCount
  as neTriangle ptr triangles
  as long           triangleCount
  as long           List ' I added this member to store a compiled OpenGL list name for rendering
end type

' add a vertex and return the vertexIndex
function neTriangleMesh . AddVertex(byval x as single, _
                                    byval y as single, _
                                    byval z as single, _
                                    byval bOnce as boolean) as integer

  if bOnce=true then 
    ' reduce the number verticies
    if vertices<>NULL andalso vertexCount>0  then
      for vertexIndex as integer = 0 to vertexCount-1
        ' if the exits vertex and the new vertex 
        ' share the same 1cm x 1cm x 1cm space 
        ' than we say its the same vertex :-)
        dim as single d = vertices[vertexIndex].x-x : if d<0 then d=-d
        if d<=0.01f then ' x
          d = vertices[vertexIndex].y-y : if d<0 then d=-d
          if d<=0.01f then ' y
            d = vertices[vertexIndex].z-z : if d<0 then d=-d
            ' z return the "same" vertex
            if d<=0.01f then return vertexIndex 
          end if
        end if
      next
    end if  
  end if
 
  ' add a "new" vertex
  var index = vertexCount : vertexCount+=1
  ' NOTE: neV3 are padded to 16 byte not 12 !
  vertices = reallocate(vertices,vertexCount*sizeof(neV3))
  vertices[index].x=x
  vertices[index].y=y
  vertices[index].z=z
  return index
end function
function neTriangleMesh . AddVertex(byref v as const neV3, _
                                    byval bOnce as boolean) as integer
  return AddVertex(v.x,v.y,v.z,bOnce) 
end function   
function neTriangleMesh . AddTriangle(byval vertexIndex0 as integer, _
                                      byval vertexIndex1 as integer, _
                                      byval vertexIndex2 as integer, _
                                      byval materialID   as integer, _
                                      byval bOnce        as boolean) as integer
  if bOnce=true then 
    ' reduce the number of triangles (don't add dupplicated triangles)
    if triangleCount>0 andalso triangles<>0 then
      for triangleIndex as integer = 0 to triangleCount-1
        ' If the existing triangle and the new triangle have different materials then they cannot be the same!
        if triangles[triangleIndex].materialID<>materialID then continue for
        ' if the exits triangle and the new triangle 
        ' use the same winding order (CCW or CW) 
        ' and same verticies they are really the same !
        if triangles[triangleIndex].indices(0)=vertexIndex0 andalso _
           triangles[triangleIndex].indices(1)=vertexIndex1 andalso _
           triangles[triangleIndex].indices(2)=vertexIndex2 then return triangleIndex
      
        if triangles[triangleIndex].indices(1)=vertexIndex0 andalso _
           triangles[triangleIndex].indices(2)=vertexIndex1 andalso _
           triangles[triangleIndex].indices(0)=vertexIndex2 then return triangleIndex
      
        if triangles[triangleIndex].indices(2)=vertexIndex0 andalso _
           triangles[triangleIndex].indices(0)=vertexIndex1 andalso _
           triangles[triangleIndex].indices(1)=vertexIndex2 then return triangleIndex      
      
      next
    end if
  end if
  ' add a "new" triangle by 3 indicies 
  var index = triangleCount : triangleCount+=1
  triangles = reallocate(triangles,triangleCount*sizeof(neTriangle))
  triangles[index].indices(0) = vertexIndex0
  triangles[index].indices(1) = vertexIndex1
  triangles[index].indices(2) = vertexIndex2
  triangles[index].materialID = materialID
  triangles[index].flag = 0 '  NE_TRI_TRIANGLE
  triangles[index].userData=NULL  
  return index
end function

function neTriangleMesh . AddTriangle(byref v0 as const neV3, _
                                      byref v1 as const neV3, _
                                      byref v2 as const neV3, _
                                      byval materialID as integer, _
                                      byval bOnce      as boolean) as integer
  var a = AddVertex(v0,bOnce)
  var b = AddVertex(v1,bOnce)
  var c = AddVertex(v2,bOnce)
  return AddTriangle(a,b,c,materialID,bOnce)
end function

' ##########################################
' # optional: user defined cdecl callbacks #
' ##########################################

' The neCustomCDInfo struct must be filled inside of the optinal 
' neCustomCDRB2RB or neCustomCDRB2AB callback's
type neCustomCDInfo
  as neV3    collisionNormal
  as neV3    worldContactPointA
  as neV3    worldContactPointB
  as single  penetrationDepth
  as long    materialIdA
  as long    materialIdB
end type

' The neBodyType are used in the neCollisionInfo structure 
' and classify the type of bodies involved in a collision callback.
type neBodyType as ulong
const as neBodyType NE_TERRAIN       = 0
const as neBodyType NE_RIGID_BODY    = 1
const as neBodyType NE_ANIMATED_BODY = 2

' This neCollisionInfo structure is passed to the application during neCollisionCallback.
' It contains various blocks of information about a collision which 
' has occurred during the current simulation step.
type neCollisionInfo
  ' bodyA and bodyB are pointers to the bodies which are colliding.
  ' Their type depends on the value of typeA and typeB.
  ' If the value of typeB is NE_TERRAIN, then bodyB is NULL.
  union 'bodyA
    as neRigidBody    ptr RigidBodyA
    as neAnimatedBody ptr AnimatedBodyA 
  end union
  union 'bodyB
    as neRigidBody    ptr RigidBodyB
    as neAnimatedBody ptr AnimatedBodyB 
  end union
  as neBodyType typeA
  as neBodyType typeB
  ' These are pointers to neGeometry objects involved in the collision, one from each of the body.
  ' If typeB equal to NE_TERRAIN, then geometryB is NULL
  as neGeometry ptr geometryA
  as neGeometry ptr geometryB
  ' The material indices of the geometries which are colliding.
  ' (remember the terrain material ID is the only material ID of -1)
  as long materialIdA
  as long materialIdB
  ' The contact points in body coordinates and in world coordinates.
  as neV3 bodyContactPointA    ' contact point A in body space of A
  as neV3 bodyContactPointB    ' contact point B in body space of B
  as neV3 worldContactPointA   ' contact point A in world space
  as neV3 worldContactPointB   ' contact point B in world space
  ' The relativeVelocity of the contact points. 
  as neV3 relativeVelocity
  ' The direction of the collision normal.
  as neV3 collisionNormal
end type

extern "c"
' optional and for advanced user only custom rigdbody to rigidbody collision callback
type neCustomCDRB2RBCallback as function (byval bodyA  as neRigidBody ptr, _
                                          byval bodyB  as neRigidBody ptr, _
                                          byref cdInfo as neCustomCDInfo) as boolean

' optional and for advanced user only custom rigdbody to animatedbody collision callback
type neCustomCDRB2ABCallback as function (byval bodyA  as neRigidBody ptr, _
                                          byval bodyB  as neAnimatedBody ptr, _
                                          byref cdInfo as neCustomCDInfo) as boolean

' optional but usefull beginner friendly collision callback
type neCollisionCallback as sub (byref collisionInfo as neCollisionInfo)

' optional and for advanced user only but powerfull custom terrain triangle query callback.
type neTerrainTriangleQueryCallback as sub (byref minBound           as const neV3, _
                                            byref maxBound           as const neV3, _
                                            byval candidateTriangles as long ptr ptr, _
                                            byval triangles          as neTriangle ptr ptr, _
                                            byval vertices           as neV3 ptr ptr, _
                                            byval candidateCount     as long ptr, _
                                            byval triangleCount      as long ptr, _
                                            byval body               as neRigidBody ptr)

' user defined beginner friendly breakage callback must be defined if the neGeometry breakage feature are used.
type neBreakageCallback as sub (byval originalBody     as any ptr, _
                                byval originalbodyType as neBodyType, _
                                byval brokenGeometry   as neGeometry ptr, _
                                byval newBody          as neRigidBody)

' optional but powerfull user defined rigidbody controller callback
type neRigidBodyControllerCallback as sub (byval controller as neRigidBodyController ptr, byval timeStep as single)

' optional but powerfull user defined joint controller callback
type neJointControllerCallback as sub (byval controller as neJointController ptr, byval timeStep as single)

' optional user defined logger message output callback
type neLogOutputCallback as sub (byval logString as const zstring ptr)
' If all the behavior of your simulation are fine you will never need 
' a neLogOutputCallback() but if something goes wrong or a total crash happents 
' you can get some usefull logger messages for the rison of the problem.

' Imagine you use the low number of default objects while creating the simulator
' and after your game or what ever app you wrote add one more rgidbody as allocated and it crashed.
' Set the callback to a sub that appends logger messages to a file.
' If the problem happents or the program chrashed take a look in the file 
' and you will see a message something like: 
' "Run out of RigidBodies. Increase 'rigidBodiesCount'"

' ##############
' # class neV3 #
' ##############
' legende: 
' = x . y is the dot product of vector x and y
' = x x y is the cross product of vector x and y
' = |x| is the length of vector x

' var v = CretaeV3(x,y,z)
declare function CreateV3(byval x as single=0.0f, byval y as single=0.0f, byval z as single=0.0f) as neV3
' dim as neV3 v : V3Set(v,x,y,z)
declare sub      V3Set(byref v as neV3, byval x as single, byval y as single, byval z as single)
' dim as neV3 v : V3Set(v,other)
declare sub      V3SetV3(byref v as neV3, byref other as const neV3)
' this=abs(this)
declare sub      V3SetAbs(byref v as neV3)
' v3 = abs(other)
declare function V3Abs(byref other as const neV3) as neV3
' this = min(a,b)
declare sub      V3SetMin(byref v as neV3,byref a as const neV3, byref b as const neV3)
' v3 = min(a,b)
declare function V3Min(byref a as const neV3, byref b as const neV3) as neV3
' this = max(a,b)
declare sub      V3SetMax(byref v as neV3,byref a as const neV3, byref b as const neV3)
' v3 = max(a,b)
declare function V3Max(byref a as const neV3, byref b as const neV3) as neV3

' v3 = -v3
declare function V3Neg(byref v as const neV3) as neV3
' a += b
declare sub      V3Inc(byref a as neV3, byref b as const neV3)
' a -= b
declare sub      V3Dec(byref a as neV3, byref b as const neV3)
' v3 = a + b
declare function V3Add(byref a as const neV3, byref b as const neV3) as neV3
' v3 = a - b
declare function V3Sub(byref a as const neV3, byref b as const neV3) as neV3
' v3 = v * scalar
declare function V3Mul(byref v as const neV3, byval scalar as single) as neV3
' v3 = v / scalar
declare function V3Div(byref v as const neV3, byval scalar as single) as neV3
' v3 = v3*v3
declare function V3MulV3(byref a as const neV3, byref b as const neV3) as neV3
' v3 = v3*m3
declare function V3MulM3(byref v as const neV3, byref m3 as const neM3) as neV3
' v3 = a + b*t
declare function V3AddMul(byref a as const neV3, byref b as const neV3,byval t as single) as neV3
' scalar = a . b
declare function V3Dot(byref a as const neV3, byref b as const neV3) as single
' v3 = a x b
declare function V3Cross(byref a as const neV3, byref b as const neV3) as neV3
' v3 = normalized(a x b)
declare function V3NormalizedCross(byref a as const neV3, byref b as const neV3) as neV3
' scalar = v . v
declare function V3LengthSquared(byref v as const neV3) as single
' scalar = |v|
declare function V3Length(byref v as const neV3) as single
' scalar = (b-a) . (b-a)
declare function V3DistanceSquared(byref a as const neV3, byref b as const neV3) as single
' scalar = |b-a|
declare function V3Distance(byref a as const neV3, byref b as const neV3) as single
' v3.Normalize()
declare sub      V3Normalize(byref thisVector as neV3)
' v3 = Normalized(v3) 
declare function V3Normalized(byref inVector as const neV3) as neV3
' rotate this vector about the X,Y or Z axis
declare sub      V3RotateX(byref thisVector as neV3, byval deg as single)
declare sub      V3RotateY(byref thisVector as neV3, byval deg as single)
declare sub      V3RotateZ(byref thisVector as neV3, byval deg as single)
' return a single rotated vector
' NOTE: to rotate a bunch of vectors use M3RotateVectors or T3RotateVectors !
declare function V3RotatedX(byref inVector as const neV3, byval deg as single) as neV3
declare function V3RotatedY(byref inVector as const neV3, byval deg as single) as neV3
declare function V3RotatedZ(byref inVector as const neV3, byval deg as single) as neV3
' ##############
' # neV3 tools #
' ##############
' get pitch angle X-axis (in deg)
declare function V3GetPitch(byref v as const neV3) as single
' get yaw angle Y-axis (in deg)
declare function V3GetYaw(byref v as const neV3) as single
' fade linear from a to b by t (t>=0<=1)
' v3 = a + (b-a)*t
declare function V3Lerp(byref a as const neV3, byref b as const neV3, byval t as single) as neV3
' v3 = a + (b-a)*0.5
declare function V3Between(byref a as const neV3, byref b as const neV3) as neV3
' this = this * (this . other)
declare sub      V3Project(byref thisVector as neV3,byref other as const neV3)
' v3 = a * (a . b)
declare function V3Projected(byref a as const neV3,byref b as const neV3) as neV3
' return the distance between a line and v
declare function V3GetDistanceToLine(byref v as const neV3,byref lineStart as const neV3,byref lineEnd as const neV3) as single
' return the distance between a line and v and the nearst point on the line
declare function V3GetDistanceAndNearestPointOnLine(byref v as const neV3,byref lineStart as const neV3,byref lineEnd as const neV3, byref outNearest as neV3) as single
' same as above but the line are definded like a ray.
declare function V3GetDistanceAndNearestPointOnRay(byref v as const neV3,byref lineStart as const neV3,byref lineDirection as const neV3, byref outNearest as neV3) as single
declare sub      V3ChooseAxis(byref outNormalizedAxis as neV3, byref outCrossAxis as neV3, byref inputNormal as const neV3)
declare sub      V3RemoveComponent(byref outVector as neV3, byval inX as single, byval inY as single, byval inZ as single)
declare sub      V3RemoveComponentV3(byref outVector as neV3, byref inComponed as const neV3)

'#######################################
'# calculate the inertia tensor vector #
'#######################################
declare function CubeInertiaTensorV3(byval fSize as single, byval fMass as single) as neV3
declare function BoxInertiaTensorV3(byval fWidth as single, byval fHeight as single, byval fDepth as single, byval fMass as single) as neV3
declare function SphereInertiaTensorV3(byval fDiameter as single, byval fMass as single) as neV3
declare function CapsuleInertiaTensorV3(byval fDiameter as single, byval fHeight as single, byval fMass as single) as neV3

' ##############
' # class neM3 #
' ##############
' create identity matrix
declare function CreateIdentityMatrix() as neM3
' create matrix from pitch yaw roll
declare function CreateRotMatrix(byval p as single, byval y as single, byval r as single) as neM3
' create matrix from pitch yaw roll vector
declare function CreateRotMatrixV3(byref pyr as const neV3) as neM3
' create matrix from quaternion
declare function CreateRotMatrixQ(byref q as const neQ) as neM3
' create matrix from Axis and angle
declare function CreateRotMatrixAxisAngle(byval axisx as single, byval axisy as single,byval axisz as single, byval angle as single) as neM3
' create matrix from Axis and angle vector
declare function CreateRotMatrixAxisAngleV3(byref axis as const neV3, byval angle as single) as neM3
declare function CreateRotXMatrix(byval deg as single) as neM3
declare function CreateRotYMatrix(byval deg as single) as neM3
declare function CreateRotZMatrix(byval deg as single) as neM3
declare function CreateRotXYZMatrix(byval xdeg as single, byval ydeg as single, byval zdeg as single) as neM3

' set identity matrix from Axis and angle vector
declare sub      M3SetIdentity(byref m3 as neM3)
' get pitch yaw roll from matrix
declare sub      M3GetRotAngle(byref m3 as const neM3, byref rp as single, byref ry as single, byref rr as single)
' get pitch yaw roll vector from matrix
declare function M3GetRotAngleV3(byref m3 as const neM3) as neV3
' v3 = m3*v3
declare function M3MulV3(byref m3 as const neM3, byref v as const neV3) as neV3
' v3 = m3.transpose()*v3
declare function M3TransposeMulV3(byref m3 as const neM3, byref v as const neV3) as neV3
' m3.transpose()
declare sub      M3SetTranspose(byref m3 as neM3)
' m3 = transpose(m3)
declare function M3GetTranspose(byref m3 as const neM3) as neM3
' m3 = m3+m3
declare function M3Add(byref a as const neM3, byref b as const neM3) as neM3
' m3 = m3-m3
declare function M3Sub(byref a as const neM3, byref b as const neM3) as neM3
' m3 = m3*scalar
declare function M3Mul(byref a as const neM3, byval scalar as single) as neM3
' m3 = m3*m3
declare function M3MulM3(byref a as const neM3, byref b as const neM3) as neM3
' NEW: m3 = m3*m3*m3
declare function M3Mul3(byref a as const neM3, byref b as const neM3, byref c as const neM3) as neM3
' NEW: m3 = m3*m3*m3*m3
declare function M3Mul4(byref a as const neM3, byref b as const neM3, byref c as const neM3,byref d as const neM3) as neM3
' fade linear from a to b by t
declare function M3Lerp(byref a as const neM3, byref b as const neM3, byval t as single) as neM3
' set the matrix inverse return false if no solution found
declare function M3SetInverse(byref m3 as neM3) as boolean
' return a the inverse of the given matrix
declare function M3GetInverse(byref m3 as const neM3) as neM3
' v3 = m3*v3
declare function M3RotateVector(byref m3 as const neM3, byref v as const neV3) as neV3
' loop n : d[n] = m3*s[n] 
declare sub      M3RotateVectors(byref m3 as const neM3, byval d as neV3 ptr, byval s as const neV3 ptr, byval n as long)
' NEW: m=inverse(m3) : loop n : d[n] = m*s[n] 
declare sub      M3InverseRotateVectors(byref m3 as const neM3, byval d as neV3 ptr, byval s as const neV3 ptr, byval n as long)

' ##############
' #class neT3 #
' ##############
declare function CreateIdentityTransform() as neT3
declare sub      T3SetIdentiy(byref t3 as neT3)
declare sub      T3SetPosition(byref t3 as neT3, byval px as single, byval py as single, byval pz as single)
declare sub      T3SetPositionV3(byref t3 as neT3, byref position as const neV3)
declare sub      T3SetRotation(byref t3 as neT3, byval rp as single, byval ry as single, byval rr as single)
declare sub      T3SetRotationV3(byref t3 as neT3, byref rotation as const neV3)
declare sub      T3SetRotationM3(byref t3 as neT3, byref rotation as const neM3)
declare sub      T3SetPositionRotation(byref t3 as neT3, byval px as single, byval py as single, byval pz as single, byval rp as single, byval ry as single, byval rr as single)
declare sub      T3SetPositionRotationV3(byref t3 as neT3, byref position as const neV3, byref rotation as const neV3)
declare sub      T3SetPositionRotationM3(byref t3 as neT3, byref position as const neV3, byref rotation as const neM3)
' v3 = T3*v3
declare function T3MulV3(byref t3 as const neT3, byref v as const neV3) as neV3
' t3 = t3*t3
declare function T3MulT3(byref a as const neT3, byref b as const neT3) as neT3
' NEW: t3 = t3*t3*t3
declare function T3Mul3(byref a as const neT3, byref b as const neT3, byref c as const neT3) as neT3
' NEW: t3 = t3*t3*t3*t3
declare function T3Mul4(byref a as const neT3, byref b as const neT3, byref c as const neT3, byref d as const neT3) as neT3
' this.inverse()
declare sub      T3SetInverse(byref t3 as neT3)
' t3=t3.inverse()
declare function T3GetInverse(byref t3 as const neT3) as neT3
' tools
declare sub      T3GetOpenGLMatrix(byref t3 as const neT3, byval mat4x4 as single ptr)
declare sub      T3GetInverseOpenGLMatrix(byref t3 as const neT3, byval mat4x4 as single ptr)
declare function T3RotateVector(byref t3 as const neT3, byref v as const neV3) as neV3
declare function T3TransformVector(byref t3 as const neT3, byref v as const neV3) as neV3
declare sub      T3RotateVectors(byref t3 as const neT3, byval d as neV3 ptr, byval s as const neV3 ptr, byval n as long)
declare sub      T3TransformVectors(byref t3 as const neT3, byval d as neV3 ptr, byval s as const neV3 ptr, byval n as long)
' NEW: 
declare sub      T3InverseRotateVectors(byref t3 as const neT3, byval d as neV3 ptr, byval s as const neV3 ptr, byval n as long)
' NEW: 
declare sub      T3InverseTransformVectors(byref t3 as const neT3, byval d as neV3 ptr, byval s as const neV3 ptr, byval n as long)
' #############
' # class neQ #
' #############
declare function CreateQuaternion(byval x as single, byval y as single, byval z as single, byval w as single) as neQ
declare function CreateQuaternionV3(byref xyz as const neV3, byval w as single) as neQ
declare function CreateQuaternionM3(byref m3 as const neM3) as neQ
declare function CreateQuaternionAxisAngle(byref axis as const neV3, byval angle as single) as neQ
declare sub      QNormalize(byref q as neQ)
declare function QNormalized(byref q as const neQ) as neQ
' q = q*q
declare function QMulQ(byref a as const neQ, byref b as const neQ) as neQ
' q = q*q*q
declare function QMul3(byref a as const neQ, byref b as const neQ, byref c as const neQ) as neQ
' q = q*q*q*q
declare function QMul4(byref a as const neQ, byref b as const neQ, byref c as const neQ, byref d as const neQ) as neQ
declare sub      QGetAxisAngle(byref q as const neQ, byref axisx as single, byref axisy as single, byref axisz as single, byref angle as single)
declare sub      QGetAxisAngleV3(byref q as const neQ, byref axis as neV3, byref angle as single)
' fade linear between a and b by t (>=0.0 <=1.0)
declare function QLerp(byref a as const neQ, byref b as const neQ, byval t as single) as neQ

' #####################
' # class neSimulator #
' #####################
' Use CreateSimulator to create a new instance of the simulator. 
' The sizeInfo structure specifies the amount of various objects to allocate within the simulator.
' The gravity vector describes the amount and direction of G.
declare function CreateSimulator(byref sizeInfo as const neSimulatorSizeInfo=neSimulatorSizeInfo(), _
                                 byval gX as single= 0.0f, _
                                 byval gY as single=-9.81f,_
                                 byval gZ as single= 0.0f) as neSimulator ptr
' the same as above but vector as arguments
declare function CreateSimulatorV3(byref sizeInfo as const neSimulatorSizeInfo=neSimulatorSizeInfo(), _
                                   byref gravity  as const neV3=neV3(0.0f,-9.81f,0.0f)) as neSimulator ptr

' Use DestroySimulator to delete an existing instance of the simulator object.
declare sub      DestroySimulator(byref sim as neSimulator ptr)

declare sub      SimulatorSetGravity(byval sim as neSimulator ptr, byval gx as single, byval gy as single, byval gz as single)
declare sub      SimulatorGetGravity(byval sim as neSimulator ptr, byref gx as single, byref gy as single, byref gz as single)
' same as above but vector as datatype
declare sub      SimulatorSetGravityV3(byval sim as neSimulator ptr, byref grav as const neV3)
declare function SimulatorGetGravityV3(byval sim as neSimulator ptr) as neV3

' Specify and retrieve the material properties for a corresponding material index (0 to 255).
' By default all 256 materials have the following values: friction = 0.4, restitution = 0.5
declare function SimulatorSetMaterial(byval sim as neSimulator ptr, byval index as long, _
                                      byval friction as single, byval restitution as single) as boolean
declare function SimulatorGetMaterial(byval sim as neSimulator ptr, byval index as long, _
                                      byref friction as single, byref restitution as single) as boolean

' Advance the simulation by the amount deltatime in seconds.
' The default value 1/60 = 60 Hz I choose is a good balanced value 
' for game physics in scope of accuracy and speed !
' It's an unwritten law for the most integrators I know that the dela timestep 
' should be constant for the whole runtime of the simulation.
' Trust me "don't use variable delta timesteps" or the simulated behavior becomes more and more unstable!
' Tip: If your simulation becomes more of complexity and runs out of time 
' don't reduce the number of calling SimulatorAdvance() per seconds 
' instead try to reduce the number of rendered frames per second.
' For example you call in a main loop every frame SimulatorAdvance() (60 times per second)
' if the simualtion time runs slower than the real clock than render your world only
' every second physics frame = 30 Hz.
' In general often you don't need to render your game world 60 times per seconds. 
' Remember 24/25 pictures per second was for movies the standard over many decades :-)
declare sub      SimulatorAdvance(byval sim as neSimulator ptr, byval delaTime as single = 1/60)
' nSteps specifies the number of sub-steps to perform.
declare sub      SimulatorAdvanceSteps(byval sim as neSimulator ptr, byval delaTime as single = 1/60, byval steps as long)
' TIP don't use it: only for really rare cases and only for advanced users !
declare sub      SimulatorAdvanceMinMax(byval sim as neSimulator ptr, byval delaTime as single = 1/60, byval minTimeStep as single, byval maxTimeStep as single)

' #####################
' # neSimulator misc. #
' #####################
type LOG_OUTPUT_LEVEL as long
const as LOG_OUTPUT_LEVEL LOG_NONE = 0
const as LOG_OUTPUT_LEVEL LOG_ONE  = 1
const as LOG_OUTPUT_LEVEL LOG_FULL = 2
declare sub      SimulatorSetLogOutputLevel(byval sim as neSimulator ptr, byval level as LOG_OUTPUT_LEVEL)
declare function SimulatorGetLogOutputLevel(byval sim as neSimulator ptr) as LOG_OUTPUT_LEVEL

' return the the neSizeInfo struct used to init the simulator.
declare function SimulatorGetStartSizeInfo(byval sim as neSimulator ptr) as neSimulatorSizeInfo
' return the the current neSizeInfo struct used to init the simulator.
declare function SimulatorGetCurrentSizeInfo(byval sim as neSimulator ptr) as neSimulatorSizeInfo

' get total number of bytes the simulator allocated
declare function SimulatorGetMemoryAllocated(byval sim as neSimulator ptr) as long
' return the number of all sleeping/rested rigidbodies
declare function SimulatorGetIdleBodyCount(byval sim as neSimulator ptr) as long

' #######################
' # optional callback's #
' #######################
' Specify and retrieve the message logger callback function.
declare sub      SimulatorSetLogOutputCallback(byval sim as neSimulator ptr, byval cb as neLogOutputCallback)
declare function SimulatorGetLogOutputCallback(byval sim as neSimulator ptr) as neLogOutputCallback

' Specify and retrieve the collision callback function.
' The collision callback function returns information about a collision between two objects in the simulation.
' Each pair of objects will only receive a maximum of one collision callback for each timestep of the simulation.
declare sub      SimulatorSetCollisionCallback(byval sim as neSimulator ptr, byval cb as neCollisionCallback)
declare function SimulatorGetCollisionCallback(byval sim as neSimulator ptr) as neCollisionCallback

' Specify and retrieve the breakage callback function.
' The breakage callback function is called by the simulator whenever a breakage is about to occur.
' The callback function is called for each geometry object which has broken.
' When the geometry is broken, a new rigid body (or rigid particle, depending the breakage flag) is spawned.
' During the breakage callback function the application can retrieve the pointer to the newly spawned object.
declare sub      SimulatorSetBreakageCallback(byval sim as neSimulator ptr, byval cb as neBreakageCallback)
declare function SimulatorGetBreakageCallback(byval sim as neSimulator ptr) as neBreakageCallback

' Specify and retrieve the neTerrainTriangleQueryCallback function.
declare sub      SimulatorSetTerrainTriangleQueryCallback(byval sim as neSimulator ptr, byval cb as neTerrainTriangleQueryCallback)
declare function SimulatorGetTerrainTriangleQueryCallback(byval sim as neSimulator ptr) as neTerrainTriangleQueryCallback

declare sub      SimulatorSetCustomCDRB2RBCallback(byval sim as neSimulator ptr, byval cb as neCustomCDRB2RBCallback)
declare function SimulatorGetCustomCDRB2RBCallback(byval sim as neSimulator ptr) as neCustomCDRB2RBCallback

declare sub      SimulatorSetCustomCDRB2ABCallback(byval sim as neSimulator ptr, byval cb as neCustomCDRB2ABCallback)
declare function SimulatorGetCustomCDRB2ABCallback(byval sim as neSimulator ptr) as neCustomCDRB2ABCallback

' ####################
' # neCollisionTable #
' ####################
type neReponseBitFlag as ulong
const as neReponseBitFlag RESPONSE_IGNORE           = 0
const as neReponseBitFlag RESPONSE_IMPULSE          = 1
const as neReponseBitFlag RESPONSE_CALLBACK         = 2
const as neReponseBitFlag RESPONSE_IMPULSE_CALLBACK = 3

' The collision table stores 64 pairs of response flags.
' It's simple an 2D array of (64,64) items of neReponseBitFlag's
' By default all pairs are set with "RESPONSE_IMPULSE"
' for i=0 to 63
'   for j=0 to 63
'     array(i,j) = RESPONSE_IMPULSE
'   next
' next

' That means for all body colision ID's 0-63 the simulator calculates for a collision the right impulse. 

' All posible combinations of bodyA and bodyB
' A is neRigidBody    and B is neRigidBody
' A is neRigidBody    and B is neAnimatedBody
' A is neRigidBody    and B is neTerrain
' A is neAnimatedBody and B is neRigidBody
' A is neAnimatedBody and B is neAnimatedBody (*) see NOTE 1
' A is neAnimatedBody and B is neTerrain
' A is neTerrain      and B is neRigidBody
' A is neTerrain      and B is neAnimatedBody

' NOTE 1: 
' You can define a pair of (neAnimatedBody, neAnimatedBody) in collision table
' the simulator self will never calculates any collision response for static bodies.
' But you can set for this kind of pair the RESPONSE_CALLBACK flag 
' to be informed via a callback that this two static bodies are overlaps / collided !

' NOTE 2: 
' There is no pair of (neTerrain, neTerrain) bcause the terrain is the one and only terrain in the simulation!

' If the collision objects (neGeometry) of both bodies collided the RESPONSE_IMPULSE will be calculated !

' If you like to disable any collision bneween bodyA and bodyB
' use: CollisionTableSet(table, bodyA_CollisionID, bodyB_CollisionID, RESPONSE_IGNORE)

' Or if you like that the simulator calculates the collision response 
' but you will be informed with a collision callback if the simulator detects a collision
' use: CollisionTableSet(table, bodyA_CollisionID, bodyB_CollisionID, RESPONSE_IMPULSE_CALLBACK)

' or if you like only to be informed about a collision via callback 
' but you don't need response calulation by the simulator.
' use: CollisionTableSet(table, bodyA_CollisionID, bodyB_CollisionID, RESPONSE_CALLBACK)

' Imagine you have 100 or more bodies it's more pricable to think about collision groups 
' instead of individiual pairs of collisions. (the size of the collision table is limited to 64 pairs also)
' define some groups like group 0,1,2,3 and use the collision ID'S 0,1,2,3 for all your bodies.

' NOTE: You can define the type of the neTerrain collision response with any other body also.
' in this case "and only this case" use a collision ID of -1 for the terrain !
' use: CollisionTableSet(table, body_CollisionID, -1, RESPONSE_TYPE)
' or : CollisionTableSet(table, -1, body_CollisionID, RESPONSE_TYPE)

' Retrieve the pointer to the collision table.
declare function SimulatorGetCollisionTable(byval sim as neSimulator ptr) as neCollisionTable ptr
' returns number of pairs in the table (dfault is 64)
declare function CollisionTableGetMaxCollisionID(byval tbl as neCollisionTable ptr) as long
' define or get the respose type for any pair in the table.
declare sub      CollisionTableSet(byval tbl as neCollisionTable ptr, _
                                   byval collisionID1 as long, byval collisionID2 as long, _
                                   byval response as neReponseBitFlag = RESPONSE_IMPULSE)
declare function CollisionTableGet(byval tbl as neCollisionTable ptr, _
                                   byval collisionID1 as long, byval collisionID2 as long) as neReponseBitFlag



' Use these functions to create or free a animated body (neAnimatedBody)
' When a animated body is freed, all their associated joints, geometries and callbacks are freed automatically.
declare function SimulatorCreateAnimatedBody(byval sim as neSimulator ptr) as neAnimatedBody ptr
declare sub      SimulatorFreeAnimatedBody(byval sim as neSimulator ptr, byref body as neAnimatedBody ptr)

' Use these functions to create rigid body or rigid particle (neRigidBody)
declare function SimulatorCreateRigidBody(byval sim as neSimulator ptr) as neRigidBody ptr
' The particle is simple a rigid body also but with one exeption it will never collide with other particles !
declare function SimulatorCreateRigidParticle(byval sim as neSimulator ptr) as neRigidBody ptr
' When a rigid body is freed, all their associated joints, 
' geometries, sensors and callbacks are freed automatically.
declare sub      SimulatorFreeRigidBody(byval sim as neSimulator ptr, byref body as neRigidBody ptr)

' Use this function to specify the terrain mesh to the simulator.
declare sub      SimulatorSetTerrainMesh(byval sim as neSimulator ptr, byval mesh as neTriangleMesh ptr)
declare sub      SimulatorFreeTerrainMesh(byval sim as neSimulator ptr)

' Active body iteration
' NEW: 
declare function SimulatorBeginIterateActiveAnimatedBody(byval sim as neSimulator ptr) as neAnimatedBody ptr
declare function SimulatorGetNextActiveAnimatedBody(byval sim as neSimulator ptr, byval ab as neAnimatedBody ptr) as neAnimatedBody ptr
' NEW: 
declare function SimulatorBeginIterateActiveRigidBody(byval sim as neSimulator ptr) as neRigidBody ptr
declare function SimulatorGetNextActiveRigidBody(byval sim as neSimulator ptr, byval rb as neRigidBody ptr) as neRigidBody ptr
' NEW: 
declare function SimulatorBeginIterateActiveRigidParticleBody(byval sim as neSimulator ptr) as neRigidBody ptr
declare function SimulatorGetNextActiveRigidParticleBody(byval sim as neSimulator ptr, byval rb as neRigidBody ptr) as neRigidBody ptr

' Inactive body iteration
' NEW: 
declare function SimulatorBeginIterateInactiveAnimatedBody(byval sim as neSimulator ptr) as neAnimatedBody ptr
declare function SimulatorGetNextInactiveAnimatedBody(byval sim as neSimulator ptr, byval ab as neAnimatedBody ptr) as neAnimatedBody ptr
' NEW: 
declare function SimulatorBeginIterateInactiveRigidBody(byval sim as neSimulator ptr) as neRigidBody ptr
declare function SimulatorGetNextInactiveRigidBody(byval sim as neSimulator ptr, byval rb as neRigidBody ptr) as neRigidBody ptr
' NEW: 
declare function SimulatorBeginIterateInactiveRigidParticleBody(byval sim as neSimulator ptr) as neRigidBody ptr
declare function SimulatorGetNextInactiveRigidParticleBody(byval sim as neSimulator ptr, byval rb as neRigidBody ptr) as neRigidBody ptr

' Create a joint that constrains the rigid body and a fixed point in the world.
declare function SimulatorCreateRigidBodyWorldJoint(byval sim as neSimulator ptr, _
                                                    byval body as neRigidBody ptr) as neJoint ptr

' Create a joint that constrains rigid body A with ridid body B
declare function SimulatorCreateRigidBodyRigidBodyJoint(byval sim as neSimulator ptr, _
                                                        byval bodyA as neRigidBody ptr, _
                                                        byval bodyB as neRigidBody ptr) as neJoint ptr
' Create a joint that constrains rigid body A with animated body B
declare function SimulatorCreateRigidBodyAnimatedBodyJoint(byval sim as neSimulator ptr, _
                                                           byval bodyA as neRigidBody ptr, _
                                                           byval bodyB as neAnimatedBody ptr) as neJoint ptr

' NOTE: Take a look for CreateJoint() in the helpers section also. (beginning at line 1450)

' When a rigid body or animated body is freed, all their associated joints, 
' geometries, sensors and callbacks are freed automatically.
' So it's really rare to free a joint manually.
declare sub SimulatorFreeJoint(byval sim as neSimulator ptr, byref joint as neJoint ptr)


' ########################
' # class neAnimatedBody #
' ########################
' Specify and retrieve the user data associate with the neAnimatedBody object.
declare sub      AnimatedBodySetUserData(byval ab as neAnimatedBody ptr, byval userData as any ptr)
declare function AnimatedBodyGetUserData(byval ab as neAnimatedBody ptr) as any ptr

' Set and retrieve spatial states for the animated body with these functions.
declare sub      AnimatedBodySetPosition  (byval ab as neAnimatedBody ptr, byval x as single, byval y as single, byval z as single)
declare sub      AnimatedBodyGetPosition  (byval ab as neAnimatedBody ptr, byref x as single, byref y as single, byref z as single)
declare sub      AnimatedBodySetPositionV3(byval ab as neAnimatedBody ptr, byref position as const neV3)
declare function AnimatedBodyGetPositionV3(byval ab as neAnimatedBody ptr) as neV3

' Set and retrieve position and roation of the animated body. (rotation in rad)
declare sub      AnimatedBodySetRotation  (byval ab as neAnimatedBody ptr, byval p as single, byval y as single, byval r as single)
declare sub      AnimatedBodyGetRotation  (byval ab as neAnimatedBody ptr, byref p as single, byref y as single, byref r as single)
declare sub      AnimatedBodySetRotationV3(byval ab as neAnimatedBody ptr, byref pyr as neV3)
declare function AnimatedBodyGetRotationV3(byval ab as neAnimatedBody ptr) as neV3

' Set and retrieve roation of the animated body by a quaternion and matrix.
declare sub      AnimatedBodySetRotationM3(byval ab as neAnimatedBody ptr, byref m as const neM3)
declare function AnimatedBodyGetRotationM3(byval ab as neAnimatedBody ptr) as neM3
declare sub      AnimatedBodySetRotationQ (byval ab as neAnimatedBody ptr, byref q as const neQ)
declare function AnimatedBodyGetRotationQ (byval ab as neAnimatedBody ptr) as neQ

' Set and retrieve position and rotation of the animated body. (rotation in deg not rad)
declare sub      AnimatedBodySetPositionRotation  (byval ab as neAnimatedBody ptr, _
                                                   byval px as single, byval py as single, byval pz as single, _
                                                   byval rp as single, byval ry as single, byval rr as single)
declare sub      AnimatedBodyGetPositionRotation  (byval ab as neAnimatedBody ptr, _
                                                   byref px as single, byref py as single, byref pz as single, _
                                                   byref rx as single, byref ry as single, byref rz as single)
' the same as above but vector as arguments
declare sub      AnimatedBodySetPositionRotationV3(byval ab as neAnimatedBody ptr, _
                                                   byref position as const neV3, _
                                                   byref rotation as const neV3)
declare sub      AnimatedBodyGetPositionRotationV3(byval ab as neAnimatedBody ptr, _
                                                   byref position as neV3, _
                                                   byref rotation as neV3)
' Set and retrieve the transform of the geometry.
declare sub      AnimatedBodySetTransform(byval ab as neAnimatedBody ptr, byref t as const neT3)
declare function AnimatedBodyGetTransform(byval ab as neAnimatedBody ptr) as neT3

' Set and retrieve the collision ID of the animated body. 
' The collision ID stored in the neCollisionTable, determines how the animated body 
' will respond when a collision has occured with other objects in the simulation.
declare sub      AnimatedBodySetCollisionID(byval ab as neAnimatedBody ptr, byval cid as long)
declare function AnimatedBodyGetCollisionID(byval ab as neAnimatedBody ptr) as long

' Use these functions to create and retrieve neGeometry objects for the animated body.
' The collection neGeometry, assigned to an animated body, defines the body's collision boundary.
' It also defines the animated body's breaking behaviour. 
' An animated body can also exist without any neGeomtry assigned to it,
' it just wont collide with anything in the simulation.
' All neGeometry objects within an animated body will be freed automatcally when the animated body is freed. 
' (See also neGeometry)
declare function AnimatedBodyAddGeometry(byval ab as neAnimatedBody ptr) as neGeometry ptr
declare function AnimatedBodyRemoveGeometry(byval ab as neAnimatedBody ptr, byval geo as neGeometry ptr) as boolean

declare function AnimatedBodyGetGeometryCount(byval ab as neAnimatedBody ptr) as long
declare sub      AnimatedBodyBeginIterateGeometry(byval ab as neAnimatedBody ptr)
declare function AnimatedBodyGetNextGeometry(byval ab as neAnimatedBody ptr) as neGeometry ptr

' Update the internal bounding information of the animated body. 
' Whenever you changes the geometry by adding, removing, or modifying the geometry, 
' this function needs to be called to update the internal bounding information.
declare sub      AnimatedBodyUpdateBoundingInfo(byval ab as neAnimatedBody ptr)

' Breaks off the geometry object, which must assigned to the animated body. Returns the newly created rigid body. 
declare function AnimatedBodyBreakGeometry(byval ab as neAnimatedBody ptr, byval geo as neGeometry ptr) as neRigidBody  ptr

' Define whether or not an animated body will collide with another body 
' indirectly connected to it via joints (neJoint).
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable. 
declare sub      AnimatedBodySetCollideConnected(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetCollideConnected(byval ab as neAnimatedBody ptr) as boolean

' Define whether or not an animated body will collide 
' with another body directly connected to it. 
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable. 
declare sub      AnimatedBodySetCollideDirectlyConnected(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetCollideDirectlyConnected(byval ab as neAnimatedBody ptr) as boolean

' advanced and optional:
declare sub      AnimatedBodySetUseCustomCollisionDetection(byval ab as neAnimatedBody ptr, _
                                                            byval yes as boolean, _
                                                            byval obb as const neT3 ptr, _
                                                            byval boundingRadius as single)
declare function AnimatedBodyGetUseCustomCollisionDetection(byval ab as neAnimatedBody ptr) as boolean

' Activate or deactivate the animated body. 
' When an animated body is deactivated, the simulator will not perform a collision test on the animated body.
' When activating (yes = true), an optional hint can be supplied.
' The hint should be a rigid body or an animated body near where the animated body being activated is.
' This can help speed up the introduction of the new activated animated body into the simulation.
declare sub      AnimatedBodySetActive(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetActive(byval ab as neAnimatedBody ptr) as boolean
declare sub      AnimatedBodySetActiveWithRigidBodyHint(byval ab as neAnimatedBody ptr, byval yes as boolean, byval hint as neRigidBody    ptr)
declare sub      AnimatedBodySetActiveWithAnimatedBodyHint(byval ab as neAnimatedBody ptr, byval yes as boolean, byval hint as neAnimatedBody ptr)

' I added some helper tools you can find a small description in section neGeometry.
declare sub      AnimatedBodyGetOpenGLMatrix(byval ab as neAnimatedBody ptr, byval m4x4 as single ptr)
declare sub      AnimatedBodyGetInverseOpenGLMatrix(byval ab as neAnimatedBody ptr, byval m4x4 as single ptr)
declare function AnimatedBodyRotateVector(byval ab as neAnimatedBody ptr, byref v as const neV3) as neV3
declare function AnimatedBodyTransformVector(byval ab as neAnimatedBody ptr, byref v as const neV3) as neV3
declare sub      AnimatedBodyRotateVectors(byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      AnimatedBodyTransformVectors(byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
' NEW: 
declare sub      AnimatedBodyInverseRotateVectors(byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
' NEW: 
declare sub      AnimatedBodyInverseTransformVectors(byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)

' #####################
' # class neRigidBody #
' #####################
' Specify and retrieve the user data associate with the neRigidBody object.
declare sub      RigidBodySetUserData(byval rb as neRigidBody ptr, byval userData as any ptr)
declare function RigidBodyGetUserData(byval rb as neRigidBody ptr) as any ptr 

' Sets and retrieves the mass of the rigid body.
' This property governs how fast a rigid body will accelerate under a given force, 
' as well as how big an impulse is needed to change the body's velocity.
declare sub      RigidBodySetMass(byval rb as neRigidBody ptr, byval mass as single)
declare function RigidBodyGetMass(byval rb as neRigidBody ptr) as single

' Set the body's mass inertia tensor. 
' This property is analagous to mass, but effects the rotational behaviour. 
' Use utility functions such as neBox-/neSphere-/neCapsule- InertiaTensorV3() to help you set this value.
declare sub      RigidBodySetInertiaTensorV3(byval rb as neRigidBody ptr, byref v as const neV3)
declare sub      RigidBodySetInertiaTensorM3(byval rb as neRigidBody ptr, byref m as const neM3)
declare sub      RigidBodyUpdateInertiaTensor(byval rb as neRigidBody ptr)

' Set and retrieve the collision ID of the rigid body.
' The collision ID, stored in the neCollisionTable, determines how the rigid body will respond 
' when a collision has occurred with other objects in the simulation. 
' See also neCollisionTable.
declare sub      RigidBodySetCollisionID(byval rb as neRigidBody ptr, byval cid as long)
declare function RigidBodyGetCollisionID(byval rb as neRigidBody ptr) as long

' Set and retrieve the linear damping factor of the rigid body. (default=0.0f)
declare sub      RigidBodySetLinearDamping(byval rb as neRigidBody ptr, byval damp as single)
declare function RigidBodyGetLinearDamping(byval rb as neRigidBody ptr) as single

' Set and retrieve the angular damping factor of the rigid body. (default=0.0f)
declare sub      RigidBodySetAngularDamping(byval rb as neRigidBody ptr, byval damp as single)
declare function RigidBodyGetAngularDamping(byval rb as neRigidBody ptr) as single

' Set and retrieve position of the rigid body.
declare sub      RigidBodySetPosition(byval rb as neRigidBody ptr, _
                                      byval px as single, byval py as single, byval pz as single)
declare sub      RigidBodyGetPosition(byval rb as neRigidBody ptr, _
                                      byref px as single, byref py as single, byref pz as single)
' the same as above but vector as arguments
declare sub      RigidBodySetPositionV3(byval rb as neRigidBody ptr, byref position as const neV3)
declare function RigidBodyGetPositionV3(byval rb as neRigidBody ptr) as neV3

' Set and retrieve pitch yaw roll rotation of the rigid body. (all values in deg not rad)
declare sub      RigidBodySetRotation(byval rb as neRigidBody ptr, _
                                      byval p as single, byval y as single, byval r as single)
declare sub      RigidBodyGetRotation(byval rb as neRigidBody ptr, _
                                      byref p as single, byref y as single, byref r as single)
' the same as above but vector as arguments
declare sub      RigidBodySetRotationV3(byval rb as neRigidBody ptr, byref rotation as neV3)
declare function RigidBodyGetRotationV3(byval rb as neRigidBody ptr) as neV3

' Set and retrieve rotation of the rigid body by neQ and neM3
declare sub      RigidBodySetRotationM3(byval rb as neRigidBody ptr, byref m as const neM3)
declare function RigidBodyGetRotationM3(byval rb as neRigidBody ptr) as neM3
declare sub      RigidBodySetRotationQ(byval rb as neRigidBody ptr, byref q as const neQ)
declare function RigidBodyGetRotationQ(byval rb as neRigidBody ptr) as neQ

' Set and retrieve position and rotation of the rigid body.
declare sub      RigidBodySetTransform(byval rb as neRigidBody ptr, byref t as const neT3)
declare function RigidBodyGetTransform(byval rb as neRigidBody ptr) as neT3
declare sub      RigidBodySetPositionRotation(byval rb as neRigidBody ptr, _
                                              byval px as single, byval py as single, byval pz as single, _
                                              byval rp as single, byval ry as single, byval rr as single)
declare sub      RigidBodyGetPositionRotation(byval rb as neRigidBody ptr, _
                                              byref px as single, byref py as single, byref pz as single,  _
                                              byref rx as single, byref ry as single, byref rz as single)
declare sub      RigidBodySetPositionRotationV3(byval rb as neRigidBody ptr, _
                                                byref position as const neV3, _
                                                byref rotation as const neV3)
declare sub      RigidBodyGetPositionRotationV3(byval rb as neRigidBody ptr, _
                                                byref position as neV3, _
                                                byref rotation as neV3)

' Set and retrieve dynamic states for the rigid body with these functions.
declare sub      RigidBodySetVelocity(byval rb as neRigidBody ptr, _
                                      byval vx as single, byval vy as single, byval vz as single)
declare sub      RigidBodyGetVelocity(byval rb as neRigidBody ptr, _
                                      byref vx as single, byref vy as single, byref vz as single)
' the same as above but vector as arguments
declare sub      RigidBodySetVelocityV3(byval rb as neRigidBody ptr, byref v as const neV3)
declare function RigidBodyGetVelocityV3(byval rb as neRigidBody ptr) as neV3

declare sub      RigidBodyGetVelocityAtPoint(byval rb as neRigidBody ptr, _
                                             byval px as single, byval py as single, byval pz as single, _
                                             byref vx as single, byref vy as single, byref vz as single)
declare function RigidBodyGetVelocityAtPointV3(byval rb as neRigidBody ptr, byref pt as const neV3) as neV3

declare sub      RigidBodyGetAngularVelocity(byval rb as neRigidBody ptr, _
                                             byref vx as single, byref vy as single, byref vz as single)
declare function RigidBodyGetAngularVelocityV3(byval rb as neRigidBody ptr) as neV3

declare sub      RigidBodySetAngularMomentum(byval rb as neRigidBody ptr, _
                                             byval mx as single, byval my as single, byval mz as single)
declare sub      RigidBodyGetAngularMomentum(byval rb as neRigidBody ptr, _
                                             byref mx as single, byref my as single, byref mz as single)
declare sub      RigidBodySetAngularMomentumV3(byval rb as neRigidBody ptr, byref am as const neV3)
declare function RigidBodyGetAngularMomentumV3(byval rb as neRigidBody ptr) as neV3

' Set and retrieve the force/torque acting on the rigid body.
' A force will accelerate the rigid body's velocity.
' A torque will accelerate the rigid body's angular velocity.
' When the force is applied at a point other than the Center of Mass of the rigid body, 
' it will also produce a torque, hence the second SetForce function, where position is defined in world coordinates.
' The force/torque will be apply continously. 
' To apply no force/torque to the rigid body set the force value to zero. 
declare sub      RigidBodySetForce(byval rb as neRigidBody ptr, _
                                   byval fx as single, byval fy as single, byval fz as single)
declare sub      RigidBodyGetForce(byval rb as neRigidBody ptr, _
                                   byref fx as single, byref fy as single, byref fz as single)
declare sub      RigidBodySetForceV3(byval rb as neRigidBody ptr, byref force as const neV3)
declare function RigidBodyGetForceV3(byval rb as neRigidBody ptr) as neV3

declare sub      RigidBodySetForceAtPosition(byval rb as neRigidBody ptr, _
                                             byval fx as single, byval fy as single, byval fz as single, _
                                             byval px as single, byval py as single, byval pz as single)
declare sub      RigidBodySetForceAtPositionV3(byval rb as neRigidBody ptr, _
                                               byref force as const neV3, _
                                               byref position as const neV3)

declare sub      RigidBodySetTorque(byval rb as neRigidBody ptr, _
                                    byval tx as single, byval ty as single, byval tz as single)
declare sub      RigidBodyGetTorque(byval rb as neRigidBody ptr, _
                                    byref tx as single, byref ty as single, byref tz as single)
' the same as above but vector as arguments
declare sub      RigidBodySetTorqueV3(byval rb as neRigidBody ptr, byref torque as const neV3)
declare function RigidBodyGetTorqueV3(byval rb as neRigidBody ptr) as neV3

' Apply an impulse or a twist to the rigid body.
' An impulse will produce a immediate change in velocity.
' A twist will produce immediate a change in angular velocity.
' An impulse applied at a point other than the Center of Mass of the rigid body also produce a twist.
' To do this, use the second ApplyImpulse function, where position is define in world coordinates of the application point.
declare sub      RigidBodyApplyImpulse(byval rb as neRigidBody ptr, _
                                       byval ix as single,byval iy as single,byval iz as single)
declare sub      RigidBodyApplyImpulseAtPosition(byval rb as neRigidBody ptr, _
                                                 byval ix as single, byval iy as single, byval iz as single, _
                                                 byval px as single, byval py as single, byval pz as single)
declare sub      RigidBodyApplyTwist(byval rb as neRigidBody ptr, _
                                     byval tx as single,byval ty as single, byval tz as single)
' the same as above but vector as arguments
declare sub      RigidBodyApplyImpulseV3(byval rb as neRigidBody ptr, byref impulse as const neV3)
declare sub      RigidBodyApplyImpulseAtPositionV3(byval rb as neRigidBody ptr, byref impulse as const neV3, byref position as const neV3)
declare sub      RigidBodyApplyTwistV3(byval rb as neRigidBody ptr, byref twist as const neV3)
' Enable or disable gravity for the rigid body. (gravity force is enabled by default)
' If gravity is enabled, the rigid body will have a force equal to gravity x mass
' automatically applied to it by the simulator.
declare sub      RigidBodyGravityEnable(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGravityIsOn(byval rb as neRigidBody ptr) as boolean

' Use these functions to create and retrieve neGeometry objects for the rigid body.
' The collection neGeometry assigned to a rigid body defines the body's collision boundary.
' It also defines the rigid body's breaking behaviour. 
' A rigid body can also exist and be simulated without any neGeomtry assigned to it; 
' it just won't collide with anything in the simulation. 
' All neGeometry objects within a rigid body will be freed automatcally when the rigid body is freed. See also neGeometry.
declare function RigidBodyAddGeometry(byval rb as neRigidBody ptr) as neGeometry ptr
declare function RigidBodyRemoveGeometry(byval rb as neRigidBody ptr, byval geo as neGeometry ptr) as boolean
declare function RigidBodyGetGeometryCount(byval rb as neRigidBody ptr) as long
declare sub      RigidBodyBeginIterateGeometry(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextGeometry(byval rb as neRigidBody ptr) as neGeometry ptr

' Update the internal bounding information of the rigid body.
' Whenever you changes the geometry and/or sensor of the rigid body 
' by adding, removing, or modifying the geometry and/or sensor, 
' this function needs to be called to update the internal bounding information.
declare sub      RigidBodyUpdateBoundingInfo(byval rb as neRigidBody ptr)

' Breaks off the geometry object, which must assigned to the rigid body. Returns the newly created rigid body.
declare function RigidBodyBreakGeometry(byval rb as neRigidBody ptr, byval geo as neGeometry ptr) as neRigidBody ptr

' Define whether or not a rigid body will collide with another body indirectly connected to it via joint (neJoint).
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable.
declare sub      RigidBodySetCollideConnected(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetCollideConnected(byval rb as neRigidBody ptr)  as boolean

' Define whether or not a rigid body will collide with another body directly connected.
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable.
declare sub      RigidBodySetCollideDirectlyConnected(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetCollideDirectlyConnected(byval rb as neRigidBody ptr) as boolean

declare sub      RigidBodySetUseCustomCollisionDetection(byval rb as neRigidBody ptr, _
                                                         byval yes as boolean, _
                                                         byval obb as const neT3 ptr, _
                                                         byval boundingRadius as single)
declare function RigidBodyGetUseCustomCollisionDetection(byval rb as neRigidBody ptr) as boolean

' sensors of the rigid body. see neSensor
declare function RigidBodyAddSensor(byval rb as neRigidBody ptr) as neSensor ptr
declare function RigidBodyRemoveSensor(byval rb as neRigidBody ptr, byval sen as neSensor ptr) as boolean
declare sub      RigidBodyBeginIterateSensor(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextSensor(byval rb as neRigidBody ptr) as neSensor ptr

' Set or retrieve the sleeping value (deafault = 0.2)
declare sub      RigidBodySetSleepingParameter(byval rb as neRigidBody ptr, byval sleepingParam as single)
declare function RigidBodyGetSleepingParameter(byval rb as neRigidBody ptr) as single

' Activate or deactivate the rigid body.
' When a rigid body is deactivated, the simulator will not update its dynamic state, 
' will not call its controllers, and the rigid body will not collide with anything.
' When activating (yes = true), an optional hint can be supplied.
' The hint should be a rigid body or an animated body near where the rigid body being activated is.
' This can help speed up the introduction of the newly-activated rigid body into the simulation.
declare sub      RigidBodySetActive(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetActive(byval rb as neRigidBody ptr) as boolean
declare sub      RigidBodySetActiveWithRigidBodyHint(byval rb as neRigidBody ptr, byval yes as boolean, byval hint as neRigidBody    ptr)
declare sub      RigidBodySetActiveWithAnimatedBodyHint(byval rb as neRigidBody ptr, byval yes as boolean, byval hint as neAnimatedBody ptr)

' NEW: check: is sleeping/resting 
declare function RigidBodyIsIdle(byval rb as neRigidBody ptr) as boolean
' NEW: if is IsIdle you casnwake it up manualy.
declare sub      RigidBodyWakeUp(byval rb as neRigidBody ptr)
' NEW: check: is it a particle or "normal" rigidbody
declare function RigidBodyIsParticle(byval rb as neRigidBody ptr) as boolean

' I added some helper tools you can find a small description in section neGeometry.
declare sub      RigidBodyGetOpenGLMatrix(byval rb as neRigidBody ptr, byval m4x4 as single ptr)
declare sub      RigidBodyGetInverseOpenGLMatrix(byval rb as neRigidBody ptr, byval m4x4 as single ptr)
declare function RigidBodyRotateVector(byval rb as neRigidBody ptr, byref v as const neV3) as neV3
declare function RigidBodyTransformVector(byval rb as neRigidBody ptr, byref v as const neV3) as neV3
declare sub      RigidBodyRotateVectors(byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      RigidBodyTransformVectors(byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
' NEW: 
declare sub      RigidBodyInverseRotateVectors(byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
' NEW: 
declare sub      RigidBodyInverseTransformVectors(byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)


' #########################
' # neRigidBodyController #
' #########################
declare function RigidBodyAddController(byval rb as neRigidBody ptr, _
                                        byval rbccb as neRigidBodyControllerCallback, _
                                        byval period as long=0) as neRigidBodyController ptr
declare function RigidBodyRemoveController(byval rb as neRigidBody ptr, _
                                           byval controller as neRigidBodyController ptr) as boolean

declare sub      RigidBodyBeginIterateController(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextController(byval rb as neRigidBody ptr) as neRigidBodyController ptr
' Use this stuff only inside a neRigidBodyControllerCallback !
declare function RigidBodyControllerGetRigidBody(byval con as neRigidBodyController ptr) as neRigidBody ptr

declare sub      RigidBodyControllerSetForce(byval con as neRigidBodyController ptr, _
                                             byval fx as single, byval fy as single, byval fz as single)
declare sub      RigidBodyControllerGetForce(byval con as neRigidBodyController ptr, _
                                             byref fx as single, byref fy as single, byref fz as single)
' same as above but vector as datatype
declare sub      RigidBodyControllerSetForceV3(byval con as neRigidBodyController ptr, _
                                               byref force as const neV3)
declare function RigidBodyControllerGetForceV3(byval con as neRigidBodyController ptr) as neV3

declare sub      RigidBodyControllerSetForceWithTorque(byval con as neRigidBodyController ptr, _
                                                       byval fx as single, byval fy as single, byval fz as single, _
                                                       byval px as single, byval py as single, byval pz as single)
' same as above but vector as datatype
declare sub      RigidBodyControllerSetForceWithTorqueV3(byval con as neRigidBodyController ptr, _
                                                         byref force as const neV3, _
                                                         byref position as const neV3)

declare sub      RigidBodyControllerSetTorque(byval con as neRigidBodyController ptr, _
                                              byval tx as single, byval ty as single, byval tz as single)
declare sub      RigidBodyControllerGetTorque(byval con as neRigidBodyController ptr, _
                                              byref tx as single, byref ty as single, byref tz as single)
' same as above but vector as datatype
declare sub      RigidBodyControllerSetTorqueV3(byval con as neRigidBodyController ptr, _
                                                byref torque as const neV3)
declare function RigidBodyControllerGetTorqueV3(byval con as neRigidBodyController ptr) as neV3


' ####################
' # class neGeometry #
' ####################
type GEOM_TYPE as ulong
const as GEOM_TYPE GEOM_POINT       = 0 ' internal use only !
const as GEOM_TYPE GEOM_LINE        = 1 ' internal use only !
const as GEOM_TYPE GEOM_TRIANGLE    = 2 ' internal use only !
const as GEOM_TYPE GEOM_BOX         = 3
const as GEOM_TYPE GEOM_SPHERE      = 4
const as GEOM_TYPE GEOM_CAPSULE     = 5
const as GEOM_TYPE GEOM_TERRAIN     = 6
const as GEOM_TYPE GEOM_CONVEXITY   = 7 ' internal use only !
const as GEOM_TYPE GEOM_CONVEXDCD   = 8 ' internal use only !

' retrieve the current collision shape type.
declare function GeometryGetType() as GEOM_TYPE

' Specify or retrieve the optional data associate with the neGeometry object.
declare sub      GeometrySetUserData(byval geo as neGeometry ptr, byval userData as any ptr)
declare function GeometryGetUserData(byval geo as neGeometry ptr) as any ptr

' define the created collision geometry as cube (a box where all dimensions are equal) 
declare sub      GeometrySetCubeSize(byval geo as neGeometry ptr, byval fSize as single)

' define the created collision geometry as box by its dimensions 
declare sub      GeometrySetBoxSize(byval geo as neGeometry ptr, byval fWidth as single, byval fHeight as single, byval fDepth as single)
' retrieve the dimensions of the box (it returns false if the collision shape isn't a box)
declare function GeometryGetBoxSize(byval geo as neGeometry ptr, byref fWidth as single, byref fHeight as single, byref fDepth as single) as boolean
' the same as above but vector as arguments
declare sub      GeometrySetBoxSizeV3(byval geo as neGeometry ptr, byref vSize as const neV3)
declare function GeometryGetBoxSizeV3(byval geo as neGeometry ptr, byref vSize as neV3) as boolean

' define the created collision geometry as sphere by its diameter. (radius*2)
declare sub      GeometrySetSphereDiameter(byval geo as neGeometry ptr, byval fDiameter as single)
' retrieve the diameter of the sphere (it returns false if the collision shape isn't a sphere)
declare function GeometryGetSphereDiameter(byval geo as neGeometry ptr, byref fDiameter as single) as boolean

' define the created collision geometry as capsule by its diameter and height.
declare sub      GeometrySetCapsule(byval geo as neGeometry ptr, byval fDiameter as single, byval fHeight as single)
' retrieve the diameter and height of the capsule (it returns false if the collision shape isn't a capsule)
declare function GeometryGetCapsule(byval geo as neGeometry ptr, byref fDiameter as single, byref fHeight as single) as boolean

' Set and retrieve the transform of the geometry.
' The transform defines the geometry's position and orientation relative to its host body object.
' The transform is defined in the host body's space.
declare sub      GeometrySetTransform(byval geo as neGeometry ptr, byref t as neT3)
declare function GeometryGetTransform(byval geo as neGeometry ptr) as neT3

' Set and retrieve position of the geometry. 
declare sub      GeometrySetPosition(byval geo as neGeometry ptr, byval x as single, byval y as single, byval z as single) 
declare sub      GeometryGetPosition(byval geo as neGeometry ptr, byref x as single, byref y as single, byref z as single) 
' the same as above but vector as arguments
declare sub      GeometrySetPositionV3(byval geo as neGeometry ptr, byref position as const neV3) 
declare function GeometryGetPositionV3(byval geo as neGeometry ptr) as neV3  

' Set and retrieve position and roation of the geometry. 
declare sub      GeometrySetPositionRotation(byval geo as neGeometry ptr, _
                                             byval px as single, byval py as single, byval pz as single, _
                                             byval rp as single, byval ry as single, byval rr as single)
declare sub      GeometryGetPositionRotation(byval geo as neGeometry ptr, _
                                             byref px as single, byref py as single, byref pz as single, _
                                             byref rx as single, byref ry as single, byref rz as single)
' the same as above but vector as arguments
declare sub      GeometrySetPositionRotationV3(byval geo as neGeometry ptr, _
                                               byref position as const neV3, _
                                               byref rotation as const neV3)
declare sub      GeometryGetPositionRotationV3(byval geo as neGeometry ptr, _
                                               byref position as neV3, _
                                               byref rotation as neV3)

' Set and retrieve the material index for the geometry.
' The material index refers to a material defined in the neSimulator using the function neSimulatorSetMaterial.
' During a collision, the materials of the two colliding geometries are used to calculate the correct collision response.
declare sub      GeometrySetMaterialIndex(byval geo as neGeometry ptr, byval index as long)
declare function GeometryGetMaterialIndex(byval geo as neGeometry ptr) as long

' ##############################
' # neGeometry break beahavior #
' ##############################
type neBreakFlag as ulong
' The geometry does not break off upon collision
const as neBreakFlag NE_BREAK_DISABLE            = 0
' Normal breakage mode. Only this geometry will break when a suitably large collision has occurred.
const as neBreakFlag NE_BREAK_NORMAL             = 1
' When this geometry breaks off, all geometries within the current object, 
' with breakflag not equal to NE_BREAK_DISABLE, will also break off.
const as neBreakFlag NE_BREAK_ALL                = 2
' When this geometry breaks off, its neighbours within a certain radius will also break off. 
' This radius is specified by neGeometrySetBreakageNeighbourRadius().
const as neBreakFlag NE_BREAK_NEIGHBOUR          = 3
' the following are the same as the above, except that they specify the new object spawned 
' from the geometry will be a rigid particle, rather than a rigid body.
const as neBreakFlag NE_BREAK_NORMAL_PARTICLE    = 4
const as neBreakFlag NE_BREAK_ALL_PARTICLE       = 5
const as neBreakFlag NE_BREAK_NEIGHBOUR_PARTICLE = 6

' Specify and retrieve the breaking behaviour of the geometry.
declare sub      GeometrySetBreakageFlag(byval geo as neGeometry ptr, byval flag as neBreakFlag)
declare function GeometryGetBreakageFlag(byval geo as neGeometry ptr) as neBreakFlag

' Specify and retrieve the mass and inertia mass tensor of the object spawned from the breakage.
declare sub      GeometrySetBreakageMass(byval geo as neGeometry ptr, byval mass as single)
declare function GeometryGetBreakageMass(byval geo as neGeometry ptr) as single

declare sub      GeometrySetBreakageInertiaTensor(byval geo as neGeometry ptr, _
                                                  byval tx as single, byval ty as single, byval tz as single)
declare sub      GeometryGetBreakageInertiaTensor(byval geo as neGeometry ptr, _
                                                  byref tx as single, byref ty as single, byref tz as single)
' the same as above but vector as arguments
declare sub      GeometrySetBreakageInertiaTensorV3(byval geo as neGeometry ptr, byref tensor as const neV3)
declare function GeometryGetBreakageInertiaTensorV3(byval geo as neGeometry ptr) as neV3

' Specify and retrieve the magnitude of the collision impulse required to break this geometry.
declare sub      GeometrySetBreakageMagnitude(byval geo as neGeometry ptr, byval mag as single)
declare function GeometryGetBreakageMagnitude(byval geo as neGeometry ptr) as single

' Specify and retrieve the amount of absorption the breaking of this geometry has.
declare sub      GeometrySetBreakageAbsorption(byval geo as neGeometry ptr, byval absorption as single)
declare function GeometryGetBreakageAbsorption(byval geo as neGeometry ptr) as single

' Specify and retrieve the radius within which other geometries within the same body will also break when a breakage occurs.
declare sub      GeometrySetBreakageNeighbourRadius(byval geo as neGeometry ptr, byval radius as single)
declare function GeometryGetBreakageNeighbourRadius(byval geo as neGeometry ptr) as single

' Specify and retrieve the breakage plane.
declare sub      GeometrySetBreakagePlane(byval geo as neGeometry ptr, _
                                          byval nx as single, byval ny as single, byval nz as single)
declare sub      GeometryGetBreakagePlane(byval geo as neGeometry ptr, _
                                          byref nx as single, byref ny as single, byref nz as single)
' the same as above but vector as arguments
declare sub      GeometrySetBreakagePlaneV3(byval geo as neGeometry ptr, byref planeNormal as const neV3)
declare function GeometryGetBreakagePlaneV3(byval geo as neGeometry ptr) as neV3

' ####################
' # neGeometry tools #
' ####################
' retrive the current geometry transform as OpenGL compatibe matrix or it's inverse.
' (the transform describes the current position and rotation of the geometry)
declare sub      GeometryGetOpenGLMatrix (byval geo as neGeometry ptr, byval m4x4 as single ptr)
declare sub      GeometryGetInverseOpenGLMatrix(byval geo as neGeometry ptr, byval m4x4 as single ptr)
' I added some tools for advaced users.
' rotate a vector by the current rotation state of the geometry 
declare function GeometryRotateVector    (byval geo as neGeometry ptr, byref v as const neV3) as neV3
' rotate and translate a vector by the current rotation and position state of the geometry 
declare function GeometryTransformVector (byval geo as neGeometry ptr, byref v as const neV3) as neV3
' same as above but you can roatate or transform a bunnch of vectors from source to destination array
declare sub      GeometryRotateVectors   (byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      GeometryTransformVectors(byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
' same as above with inverse rotation or transform !
declare sub      GeometryInverseRotateVectors   (byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      GeometryInverseTransformVectors(byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)


' ##################
' # class neSensor #
' ##################
' Specify and retrieve the user data associate with the neSensor object.
declare sub      SensorSetUserData(byval sen as neSensor ptr, byval userData as any ptr)
declare function SensorGetUserData(byval sen as neSensor ptr) as any ptr
' Specify the position and end position vector of the line representing the sensor object.
declare sub      SensorSetLineSensor(byval sen as neSensor ptr, _
                                     byval posX as single, byval posY as single, byval posZ as single, _
                                     byval dirX as single, byval dirY as single, byval dirZ as single)
' same as above but vector as datatype
declare sub      SensorSetLineSensorV3(byval sen as neSensor ptr, _
                                       byref position  as const neV3, _
                                       byref direction as const neV3)
' Retrieve the current position and direction of the sensor.
declare sub      SensorGetLinePosition(byval sen as neSensor ptr, _
                                       byref dirX as single, byref dirY as single, byref dirZ as single)
declare sub      SensorGetLineVector(byval sen as neSensor ptr, _
                                     byref dirX as single, byref dirY as single, byref dirZ as single)
#define SensorGetLineDirection SensorGetLineVector

' same as above but vector as data type
declare function SensorGetLinePositionV3(byval sen as neSensor ptr) as neV3
declare function SensorGetLineVectorV3  (byval sen as neSensor ptr) as neV3
#define SensorGetLineDirectionV3 SensorGetLineVectorV3

' get the line normalized direction vector
declare sub      SensorGetLineUnitVector(byval sen as neSensor ptr, _
                                         byref nx as single, byref ny as single, byref nz as single)
declare function SensorGetLineUnitVectorV3(byval sensor as neSensor ptr) as neV3
#define SensorGetNormaizedDirectionV3 SensorGetLineUnitVectorV3
' #################################
' # Sensor collision informations #
' #################################
' NOTE: sensor collision informations are only available inside a controller callback !
' If GetDetectDepth() is <= 0.0, then the sensor does not intersect with other geometries
' and all other values returned by GetDetectXXX functions are invalid.
declare function SensorGetDetectDepth(byval sen as neSensor ptr) as single

' Use these functions to retrieve the current intersection information of the sensor.
declare sub      SensorGetDetectNormal(byval sen as neSensor ptr, _
                                       byref nx as single, byref ny as single, byref nz as single)
' same as above but vector as datatype
declare function SensorGetDetectNormalV3(byval sen as neSensor ptr) as neV3

declare sub      SensorGetDetectContactPoint(byval sen as neSensor ptr, _
                                             byref cx as single, byref cy as single, byref cz as single)
' same as above but vector as datatype
declare function SensorGetDetectContactPointV3(byval sen as neSensor ptr) as neV3

declare function SensorGetDetectRigidBody   (byval sen as neSensor ptr) as neRigidBody ptr
declare function SensorGetDetectAnimatedBody(byval sen as neSensor ptr) as neAnimatedBody ptr
declare function SensorGetDetectMaterial    (byval sen as neSensor ptr) as long


' #################
' # class neJoint #
' #################
' Specify and retrieve the type of the joint
type neJointType as long
const as neJointType NE_BALLSOCKET = 0
const as neJointType NE_HINGE      = 1
const as neJointType NE_SLIDER     = 2 ' not (full) implemented !

declare sub      JointSetType(byval jo as neJoint ptr, byval t as neJointType)
declare function JointGetType(byval jo as neJoint ptr) as neJointType

' Specify and retrieve the user data associate with the neJoint object.
declare sub      JointSetUserData(byval jo as neJoint ptr, byval userData as any ptr)
declare function JointGetUserData(byval jo as neJoint ptr) as any ptr

' Retrieve the bodies which are connected by this joint.
' The bodies are specified when the joint is created and cannot be changed.
declare function JointGetRigidBodyA   (byval jo as neJoint ptr) as neRigidBody ptr
declare function JointGetRigidBodyB   (byval jo as neJoint ptr) as neRigidBody ptr
declare function JointGetAnimatedBodyB(byval jo as neJoint ptr) as neAnimatedBody ptr

' Use this function to specify the joint's frame in world coordinate. 
' To use this function, typically the two bodies of the joint would already be positioned correctly 
' in relation to each other in world co-ordinates.

' For ball-and-socket joints, the position of the frame 
' specifies the location where the two bodies are connected.

' For hinge joints, the position specifies the location of the joint, 
' and the Y-axis of the joint frame is the axis of the joint 
' (which is the axis the two bodies are free to rotate about).
declare sub      JointSetFrameWorld(byval joint as neJoint ptr, byref frame as const neT3)
declare sub      JointSetPositionRotationFrameWorld(byval jo as neJoint ptr, _
                                                    byval px as single, byval py as single, byval pz as single, _ 
                                                    byval rp as single=0.0f, byval ry as single=0.0f, byval rr as single=0.0f)
' same as above but vector as datatype
declare sub      JointSetPositionRotationFrameWorldV3(byval jo as neJoint ptr, _
                                                      byref position as const neV3, _
                                                      byref rotation as const neV3=neV3(0.0f,0.0f,0.0f))

' Use these functions to specify and retrieve the reference frame of the joint.
' The reference frame of the joint specifies position and orientation 
' of the joint with respect to each body's local frame.
declare sub      JointSetFrameA(byval jo as neJoint ptr, byref frameA as const neT3)
declare function JointGetFrameA(byval jo as neJoint ptr) as neT3

declare sub      JointSetPositionRotationFrameA(byval jo as neJoint ptr, _
                                                byval px as single, byval py as single, byval pz as single, _
                                                byval rp as single=0.0f, byval ry as single=0.0f, byval rr as single=0.0f)
' same as above but vector as datatype
declare sub      JointSetPositionRotationFrameAV3(byval jo as neJoint ptr, _
                                                  byref position as const neV3, _
                                                  byref rotation as const neV3=neV3(0.0f,0.0f,0.0f))

declare sub      JointSetFrameB(byval jo as neJoint ptr, byref frameB as const neT3)
declare function JointGetFrameB(byval jo as neJoint ptr) as neT3

declare sub      JointSetPositionRotationFrameB(byval jo as neJoint ptr, _
                                                byval px as single, byval py as single, byval pz as single, _
                                                byval rp as single=0.0f, byval ry as single=0.0f, byval rr as single=0.0f)
' same as above but vector as datatype
declare sub      JointSetPositionRotationFrameBV3(byval jo as neJoint ptr, _
                                                  byref position as const neV3, _
                                                  byref rotation as const neV3)

' Specify and retrieve the length of the joint.
' Some joints, such as hinge joints, have length as well as position and orientation.
declare sub      JointSetJointLength(byval jo as neJoint ptr, byval length as single)
declare function JointGetJointLength(byval jo as neJoint ptr) as single

' Enable or disable the joint. The second function returns true if the joint is enabled.
declare sub      JointEnable   (byval jo as neJoint ptr, byval yes as boolean)
declare function JointIsEnabled(byval jo as neJoint ptr) as boolean
' Specify and retrieve the optinal damping value of a joint.
declare sub      JointSetDampingFactor(byval jo as neJoint ptr, byval factor as single)
declare function JointGetDampingFactor(byval jo as neJoint ptr) as single

' These functions relate to the accuracy of the solution of the joint.
' Generally, lower accuracy results in faster simulation, 
' but may be prone to side effects if the accuracy is too low.
' Epsilon specifies the early-exit theshold for the constraint solver.
' Iteration relates to the number of iterations the solver will use to solve the constraint.
' Setting the epsilon to 0 means the solver will always perform 
' the specified number of iterations and will not exit early.
declare sub      JointSetEpsilon(byval jo as neJoint ptr, byval epsilon as single)
declare function JointGetEpsilon(byval jo as neJoint ptr) as single
' Optional set the number of iterations to solve the joint constraints. (default are 4 iteraions per onesimualtion step)
declare sub      JointSetIteration(byval jo as neJoint ptr, byval iteration as long)
declare function JointGetIteration(byval jo as neJoint ptr) as long

' Set the upper and lower limits of the joint.
' Joint limits are used to restrict the movement of a joint.

' For NE_BALLSOCKET and NE_HINGE joints, the joint's current angle 
' is the angle between the X-axes of the joint's frame in each of the joint bodies.

' For NE_BALLSOCKET joints, the valid range of the lower limit is 0 to PI; 
' for the upper limit, between lower the limit and PI. 

' For a NE_HINGE joint, the valid range for lower and upper limits is -PI..PI, 
' but the upper limit must be greater than the lower limit.

' For NE_BALLSOCKET joints, a twist limit can also be specified. 
' This is done by calling the SetUpperLimit2 and SetLowerLimit2 functions.
declare sub      JointSetUpperLimit(byval jo as neJoint ptr, byval limit as single)
declare function JointGetUpperLimit(byval jo as neJoint ptr) as single
declare sub      JointSetLowerLimit(byval jo as neJoint ptr, byval limit as single)
declare function JointGetLowerLimit(byval jo as neJoint ptr) as single
' NE_BALLSOCKET joint twist limits
declare sub      JointSetUpperLimit2(byval jo as neJoint ptr, byval limit as single)
declare function JointGetUpperLimit2(byval jo as neJoint ptr) as single
declare sub      JointSetLowerLimit2(byval jo as neJoint ptr, byval limit as single)
declare function JointGetLowerLimit2(byval jo as neJoint ptr) as single

' Enable or disable the joint limit. 
' The second function returns true if the joint limit is enabled. 
' The last two functions relate to the second limit for the joint. 
' Note: currently, only the NE_BALLSOCKET joints have a second limit, which is the twist. 
declare sub      JointLimitEnable    (byval jo as neJoint ptr, byval yes as boolean)
declare function JointIsLimitEnabled (byval jo as neJoint ptr) as boolean
declare sub      JointLimit2Enable   (byval jo as neJoint ptr, byval yes as boolean) 
declare function JointIsLimit2Enabled(byval jo as neJoint ptr) as boolean

' ####################
' # HingeJoint motor #
' ####################
' The axis of a NE_HINGE joint can act as an motor !
' If the NE_HINGE joint are created and was correctly positioned while connected to the bodies
' set the desired motor velocity and max motor force and enable the joint motor.
' Same was I said about the line sensor, a joint motor sounds not spectacular 
' but the number of virtual machines you can create with it is unlimited and much fun !
declare sub      JointMotorEnable(byval jo as neJoint ptr, byval yes as boolean)
declare function JointIsMotorEnabled(byval jo as neJoint ptr) as boolean
declare sub      JointSetMotor(byval jo as neJoint ptr, byval desireVelocity as single, byval maxForce as single)
declare sub      JointGetMotor(byval jo as neJoint ptr, byref desireVelocity as single, byref maxForce as single)

#if 0 ' make it sense to implement it ?
declare sub      JointGetOpenGLMatrix (byval jo as neJoint ptr, byval m4x4 as single ptr)
declare sub      JointGetInverseOpenGLMatrix(byval jo as neJoint ptr, byval m4x4 as single ptr)
declare function JointRotateVector    (byval jo as neJoint ptr, byref v as const neV3) as neV3
declare function JointTransformVector (byval jo as neJoint ptr, byref v as const neV3) as neV3
declare sub      JointRotateVectors   (byval jo as neJoint ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      JointTransformVectors(byval jo as neJoint ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      JointInverseRotateVectors   (byval jo as neJoint ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      JointInverseTransformVectors(byval jo as neJoint ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
#endif


' #####################
' # neJointController #
' #####################
' These functions provide access to adding and removing joint controller callbacks.
declare function JointAddController(byval jo as neJoint ptr, _
                                    byval cb as neJointControllerCallback, _
                                    byval period as long=0) as neJointController ptr
declare function JointRemoveController(byval jo as neJoint ptr, _
                                       byval ctrl as neJointController ptr) as boolean

declare sub      JointBeginIterateController(byval jo as neJoint ptr)
declare function JointGetNextController(byval jo as neJoint ptr) as neJointController ptr

' #############################
' # neJointControllerCallback #
' #############################
' ! IMPORTANT ! use this only inside of a neJointControllerCallback() !
declare function JointControllerGetJoint(byval controller as neJointController ptr) as neJoint ptr

declare sub      JointControllerSetForceBodyAV3(byval controller as neJointController ptr, _
                                                byref force as const neV3)
declare function JointControllerGetForceBodyAV3(byval ctrl as neJointController ptr) as neV3

declare sub      JointControllerSetForceBodyBV3(byval controller as neJointController ptr, _
                                                byref force as const neV3)
declare function JointControllerGetForceBodyBV3(byval ctrl as neJointController ptr) as neV3

declare sub      JointControllerSetTorqueBodyAV3(byval controller as neJointController ptr, _
                                                 byref torque as const neV3)
declare function JointControllerGetTorqueBodyAV3(byval ctrl as neJointController ptr) as neV3

declare sub      JointControllerSetTorqueBodyBV3(byval controller as neJointController ptr, _
                                                 byref torque as const neV3)
declare function JointControllerGetTorqueBodyBV3(byval ctrl as neJointController ptr) as neV3

declare sub      JointControllerSetForceWithTorqueBodyAV3(byval controller as neJointController ptr, _
                                                          byref force as const neV3, _
                                                          byref position as const neV3)
declare sub      JointControllerSetForceWithTorqueBodyBV3(byval controller as neJointController ptr, _
                                                          byref force as const neV3, _
                                                          byref position as const neV3)
end extern

'################################################
'#                                              #
'#  #   #  ####  #    ####  ####  ####    ####  #
'#  #   # #     #    #   # #     #   #  #      #
'#  #####  ###   #    ####  ###   ####    ###   #
'#  #   #  #     #    #     #     #   #      #  #
'#  #   #  ####  #### #     ####  #   #  ####   #
'#                                              # 
'################################################


' #######################
' # neSimulator helpers #
' #######################
' How I remember the order of all this arguments ?
' [R]on [A]nd [G]eorge [S]inging [J]ail [P]unk [C]ontinuously :lol:
function CreateSimulatorEx(byval RigidBodies    as long, _
                           byval AnimatedBodies as long=0, _
                           byval Geometries     as long=0, _ ' NOTE ALL neGeometry: from RigidBodies AnimatedBodies RigidParticels !
                           byval Sensors        as long=0, _
                           byval Joints         as long=0, _
                           byval Particles      as long=0, _
                           byval Controllers    as long=0, _ ' NOTE ALL neController: from RigidBodies + Joints
                           byval GravityX       as single= 0.0, _
                           byval GravityY       as single=-9.81, _
                           byval GravityZ       as single= 0.0) as neSimulator ptr
  dim as neSimulatorSizeInfo SizeInfo
  
  with SizeInfo
  if RigidBodies>.rigidBodiesCount then
    .rigidBodiesCount = RigidBodies
  end if
  if AnimatedBodies>.animatedBodiesCount then
    .animatedBodiesCount = AnimatedBodies
  end if  
  if Geometries>.geometriesCount then
    .geometriesCount = Geometries
  end if 
  if Sensors>.sensorsCount then
    .sensorsCount = Sensors
  end if  
  if Joints>.constraintsCount then
    .constraintsCount = Joints
  end if
  if Particles>.rigidParticleCount then
    .rigidParticleCount = Particles
  end if 
  if Controllers>.controllersCount then
    .controllersCount = Controllers
  end if
  dim as long n = .rigidBodiesCount  + .animatedBodiesCount + .rigidParticleCount
  if n>.geometriesCount then 
    .geometriesCount=n
  end if  
    n = .rigidBodiesCount + .animatedBodiesCount
    .overlappedPairsCount = (n*(n-1))\2
    ' limit to 20,000,000 million !
    if .overlappedPairsCount>20000000 then .overlappedPairsCount=20000000
  end with
  return CreateSimulator(SizeInfo,GravityX,GravityY,GravityZ)
end function
function CreateSimulatorExV3(byval RigidBodies    as long, _
                             byval AnimatedBodies as long=0, _
                             byval Geometries     as long=0, _
                             byval Sensors        as long=0, _
                             byval Joints         as long=0, _
                             byval Particles      as long=0, _
                             byval Controllers    as long=0, _
                             byref vGravity       as neV3=neV3(0.0f,-9.81f,0.0f))  as neSimulator ptr
  return CreateSimulatorEx(RigidBodies, _
                           AnimatedBodies, _
                           Geometries,_
                           Sensors, _
                           Joints, _
                           Particles, _
                           Controllers, _
                           vGravity.x, _
                           vGravity.y, _
                           vGravity.z)                             
end function                             

' ###################
' # neJoint helpers #
' ###################
function CreateJoint overload (byval sim as neSimulator ptr, _
                               byval body as neRigidBody ptr) as neJoint ptr
  if sim=NULL then return NULL                               
  return SimulatorCreateRigidBodyWorldJoint(sim, body)
end function
function CreateJoint (byval sim as neSimulator ptr, _
                      byval bodyA as neRigidBody ptr, _
                      byval bodyB as neRigidBody ptr) as neJoint ptr
  if sim=NULL then return NULL                      
  return SimulatorCreateRigidBodyRigidBodyJoint(sim,bodyA,bodyB)
end function

function CreateJoint (byval sim as neSimulator ptr, _
                      byval bodyA as neRigidBody ptr, _
                      byval bodyB as neAnimatedBody ptr) as neJoint ptr
  if sim=NULL then return NULL                      
  return SimulatorCreateRigidBodyAnimatedBodyJoint(sim,bodyA,bodyB)
end function
function CreateJoint (byval sim as neSimulator ptr, _
                      byval bodyA as neAnimatedBody ptr, _
                      byval bodyB as neRigidBody ptr) as neJoint ptr
  if sim=NULL then return NULL
  return SimulatorCreateRigidBodyAnimatedBodyJoint(sim,bodyB,bodyA)
end function

' #############################
' # neJointController helpers #
' #############################
sub JointControllerSetForceWithTorqueBodyA(byval controller as neJointController ptr, _
                                           byval fx as single,byval fy as single,byval fz as single, _
                                           byval px as single,byval py as single,byval pz as single)
  if controller=NULL then exit sub
  var force = CreateV3(fx,fy,fz)
  var position = CreateV3(px,py,pz)
  JointControllerSetForceWithTorqueBodyAV3(controller,force,position)
end sub
sub JointControllerSetForceWithTorqueBodyB(byval controller as neJointController ptr, _
                                           byval fx as single,byval fy as single,byval fz as single, _
                                           byval px as single,byval py as single,byval pz as single)
  if controller=NULL then exit sub
  var force = CreateV3(fx,fy,fz)
  var position = CreateV3(px,py,pz)
  JointControllerSetForceWithTorqueBodyBV3(controller,force,position)
end sub
' ##########################
' # neAnimatedBody helpers #
' ##########################
' create a static massless body with a box collision geometry
function CreateAnimatedBodyBox(byval sim     as neSimulator ptr, _
                               byval fWidth  as single=1, _
                               byval fHeight as single=1, _
                               byval fDepth  as single=1) as neAnimatedBody ptr
  if sim=NULL then return NULL 
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fWidth <0.01f then fWidth =0.01f
  if fHeight<0.01f then fHeight=0.01f
  if fDepth <0.01f then fDepth =0.01f 
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function
' create a static massless body with a box collision geometry
function CreateAnimatedBodyCube(byval sim   as neSimulator ptr, _
                                byval fSize as single = 1) as neAnimatedBody ptr
  if sim=NULL then return NULL                                
  return CreateAnimatedBodyBox(sim,fSize,fSize,fSize)
end function  
' create a static massless body with a sphere collision geometry
function CreateAnimatedBodySphere(byval sim       as neSimulator ptr, _
                                  byval fDiameter as single=1) as neAnimatedBody ptr
  if sim=NULL then return NULL                                  
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fDiameter <0.01f then fDiameter=0.01f
  GeometrySetSphereDiameter(geo, fDiameter)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function
' create a static massless body with a capsule collision geometry
function CreateAnimatedBodyCapsule(byval sim       as neSimulator ptr, _
                                   byval fDiameter as single=1.0f, _
                                   byval fHeight   as single=0.5f) as neAnimatedBody ptr
  if sim=NULL then return NULL                                   
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fDiameter<0.01f then fDiameter=0.01f
  if fHeight  <0.01f then fHeight = 0.01f
  GeometrySetCapsule(geo, fDiameter, fHeight)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function

' #######################
' # neRigidBody helpers #
' #######################
' create a dynamic body with mass and a box collision geometry
function CreateRigidBodyBox(byval sim     as neSimulator ptr, _
                            byval fWidth  as single=1, _
                            byval fHeight as single=1, _
                            byval fDepth  as single=1, _
                            byval fMass   as single=1) as neRigidBody ptr
  if sim=NULL then return NULL                            
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fWidth <0.01f then fWidth =0.01f
  if fHeight<0.01f then fHeight=0.01f
  if fDepth <0.01f then fDepth =0.01f
  if fMass  <1.0f  then fMass  =1.0f
  
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  RigidBodyUpdateBoundingInfo(rb)
  RigidBodySetInertiaTensorV3(rb,BoxInertiaTensorV3(fWidth,fHeight,fDepth,fMass))
  RigidBodySetMass(rb,fMass)
  
  return rb
end function
' create a dynamic body with mass and a box collision geometry
function CreateRigidBodyCube(byval sim   as neSimulator ptr, _
                             byval fSize as single=1.0f, _
                             byval fMass as single=1.0f) as neRigidBody ptr
  if sim=NULL then return NULL                             
  return CreateRigidBodyBox(sim,fSize,fsize,fSize,fMass)
end function  
' create a dynamic body with mass and a sphere collision geometry
function CreateRigidBodySphere(byval sim       as neSimulator ptr, _
                               byval fDiameter as single=1.0f, _
                               byval fMass     as single=1.0f) as neRigidBody ptr
  if sim=NULL then return NULL                               
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter<0.01f then fDiameter = 0.01f
  if fMass   <1.0f   then fMass     = 1.0f
  GeometrySetSphereDiameter(geo, fDiameter)
  RigidBodySetInertiaTensorV3(rb,SphereInertiaTensorV3(fDiameter,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function
' create a dynamic body with mass and a capsule collision geometry
function CreateRigidBodyCapsule(byval sim       as neSimulator ptr, _
                                byval fDiameter as single=1.0f, _
                                byval fHeight   as single=0.5f, _
                                byval fMass     as single=1.0f) as neRigidBody ptr
  if sim=NULL then return NULL                                
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter < 0.01f then fDiameter = 0.01f
  if fHeight   < 0.01f then fHeight   = 0.01f
  if fMass     < 1.0f  then fMass     = 1.0f
  GeometrySetCapsule(geo, fDiameter, fHeight)
  RigidBodySetInertiaTensorV3(rb,CapsuleInertiaTensorV3(fDiameter,fHeight,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

' ###############################
' # neRididParticleBody helpers #
' ###############################
' create a dynamic particle with mass and a box collision geometry
function CreateRigidParticleBox(byval sim     as neSimulator ptr, _
                                byval fWidth  as single=1.0f, _
                                byval fHeight as single=1.0f, _
                                byval fDepth  as single=1.0f, _
                                byval fMass   as single=1) as neRigidBody ptr
  if sim=NULL then return NULL                                
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fWidth <0.01f then fWidth =0.01f
  if fHeight<0.01f then fHeight=0.01f
  if fDepth <0.01f then fDepth =0.01f
  if fMass  <1.0f  then fMass  =1.0f
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  RigidBodySetInertiaTensorV3(rb,BoxInertiaTensorV3(fWidth,fHeight,fDepth,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function
' create a dynamic particle with mass and a box collision geometry
function CreateRigidParticleCube(byval sim   as neSimulator ptr, _
                                 byval fSize as single=1.0f, _
                                 byval fMass as single=1.0f) as neRigidBody ptr
  if sim=NULL then return NULL                                 
  return CreateRigidParticleBox(sim,fSize,fSize,fSize,fMass)
end function
' create a dynamic particle with mass and a sphere collision geometry
function CreateRigidParticleSphere(byval sim       as neSimulator ptr, _
                                   byval fDiameter as single=1.0f, _
                                   byval fMass     as single=1.0f) as neRigidBody ptr
  if sim=NULL then return NULL                                   
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter < 0.01f then fDiameter = 0.01f
  if fMass     < 1.0f  then fMass     = 1.0f
  GeometrySetSphereDiameter(geo, fDiameter)
  RigidBodySetInertiaTensorV3(rb,SphereInertiaTensorV3(fDiameter,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function
' create a dynamic particle with mass and a capsule collision geometry
function CreateRigidParticleCapsule(byval sim       as neSimulator ptr, _
                                    byval fDiameter as single=0.5f, _
                                    byval fHeight   as single=0.5f, _
                                    byval fMass     as single=1.0f) as neRigidBody ptr
  if sim=NULL then return NULL                                    
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter < 0.01f then fDiameter = 0.01f
  if fHeight   < 0.01f then fHeight   = 0.01f
  if fMass     < 1.0f  then fMass     = 1.0f
  GeometrySetCapsule(geo, fDiameter, fHeight)
  RigidBodySetInertiaTensorV3(rb,CapsuleInertiaTensorV3(fDiameter,fHeight,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function
